@@ -117,7 +117,8 @@ void expression_with_params_example() // since 1.3.0
 
     auto expr = jmespath::make_expression<json>("results[*].[name, uuid, $hostname]");
 
-    auto result = expr.evaluate(doc, {{"hostname", json{"localhost"}}});
+    std::map<std::string,json> params = {{"hostname", "localhost"}};
+    auto result = expr.evaluate(doc, params);
 
     auto options = jsoncons::json_options{}
         .array_array_line_splits(jsoncons::line_split_kind::same_line);
