@@ -1,11 +1,9 @@
-// Copyright 2013-2024 Daniel Parker
+// Copyright 2013-2025 Daniel Parker
 // Distributed under Boost license
 
 #include <jsoncons/json.hpp>
 #include <iostream>
 
-#if JSONCONS_VERSION_MAJOR == 0 && JSONCONS_VERSION_MINOR < 179
-
 void incremental_parsing_example()
 {
     jsoncons::json_decoder<jsoncons::json> decoder;
@@ -43,51 +41,6 @@ void incremental_parsing_example()
     }
 }
 
-#else
-
-void incremental_parsing_example()
-{
-    std::vector<std::string> chunks = {"[fal", "se,", "9", "0]"};
-    std::size_t index = 0;
-
-    auto read_chunk = [&](jsoncons::parser_input& input, std::error_code& /*ec*/) -> bool
-    {
-        if (index < chunks.size())
-        {
-            input.set_buffer(chunks[index].data(), chunks[index].size());
-            ++index;
-            return true;
-        }
-        else
-        {
-            return false;
-        }
-    };
-
-    jsoncons::json_decoder<jsoncons::json> decoder;
-    jsoncons::json_parser parser{read_chunk};
-
-    parser.reset();
-    try
-    {
-        parser.parse_some(decoder);
-        std::cout << "(1) done: " << std::boolalpha << parser.done() << ", source_exhausted: " << parser.source_exhausted() << "\n\n";
-        parser.finish_parse(decoder);
-        std::cout << "(2) done: " << std::boolalpha << parser.done() << ", source_exhausted: " << parser.source_exhausted() << "\n\n";
-        parser.check_done();
-        std::cout << "(3) done: " << std::boolalpha << parser.done() << ", source_exhausted: " << parser.source_exhausted() << "\n\n";
-
-        jsoncons::json j = decoder.get_result();
-        std::cout << "(4) " << j << "\n\n";
-    }
-    catch (const jsoncons::ser_error& e)
-    {
-        std::cout << e.what() << '\n';
-    }
-}
-
-#endif
-
 void parse_nan_replacement_example()
 {
     std::string s = R"(
@@ -106,11 +59,7 @@ void parse_nan_replacement_example()
     jsoncons::json_parser parser(options);
     try
     {
-#if JSONCONS_VERSION_MAJOR == 0 && JSONCONS_VERSION_MINOR < 179
-        parser.update(s);     // until 1.0.0
-#else        
-        parser.set_buffer(s.data(), s.size());   // since 1.0.0
-#endif        
+        parser.update(s.data(), s.size());   // since 1.0.0
         parser.parse_some(decoder);
         parser.finish_parse(decoder);
         parser.check_done();
