@@ -1,4 +1,4 @@
-// Copyright 2013-2024 Daniel Parker
+// Copyright 2013-2025 Daniel Parker
 // Distributed under the Boost license, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
@@ -8,8 +8,8 @@
 #define JSONCONS_DETAIL_PARSE_NUMBER_HPP
 
 #include <cctype>
-#include <exception>
-#include <limits> // std::numeric_limits
+#include <cstddef>
+#include <cstdint>
 #include <locale>
 #include <stdexcept>
 #include <string>
@@ -17,8 +17,10 @@
 #include <type_traits> // std::enable_if
 #include <vector>
 
+#include <jsoncons/config/compiler_support.hpp>
 #include <jsoncons/config/jsoncons_config.hpp>
 #include <jsoncons/json_exception.hpp>
+#include <jsoncons/utility/extension_traits.hpp>
 
 namespace jsoncons { namespace detail {
 
