@@ -1,4 +1,4 @@
-// Copyright 2013-2024 Daniel Parker
+// Copyright 2013-2025 Daniel Parker
 // Distributed under the Boost license, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
@@ -7,19 +7,24 @@
 #ifndef JSONCONS_DETAIL_WRITE_NUMBER_HPP
 #define JSONCONS_DETAIL_WRITE_NUMBER_HPP
 
+#include <clocale>
 #include <cmath>
-#include <exception>
+#include <cstddef>
+#include <cstdint>
 #include <limits> // std::numeric_limits
 #include <locale>
 #include <stdexcept>
 #include <stdio.h> // snprintf
 #include <string>
+#include <type_traits>
 
+#include <jsoncons/config/compiler_support.hpp>
 #include <jsoncons/config/jsoncons_config.hpp>
 #include <jsoncons/detail/grisu3.hpp>
 #include <jsoncons/detail/parse_number.hpp>
-#include <jsoncons/utility/extension_traits.hpp>
+#include <jsoncons/json_exception.hpp>
 #include <jsoncons/json_options.hpp>
+#include <jsoncons/utility/extension_traits.hpp>
 
 namespace jsoncons { 
 namespace detail {
