@@ -178,11 +178,11 @@ namespace jsoncons {
                            std::error_code& ec)
         {
             encoder.begin_array(2,semantic_tag::none,ser_context(),ec);
-            if (ec) {return;}
+            if (JSONCONS_UNLIKELY(ec)) {return;}
             encode_traits<T1,CharT>::encode(val.first, encoder, proto, ec);
-            if (ec) {return;}
+            if (JSONCONS_UNLIKELY(ec)) {return;}
             encode_traits<T2,CharT>::encode(val.second, encoder, proto, ec);
-            if (ec) {return;}
+            if (JSONCONS_UNLIKELY(ec)) {return;}
             encoder.end_array(ser_context(),ec);
         }
     };
@@ -204,7 +204,7 @@ namespace jsoncons {
                                std::error_code& ec)
             {
                 encode_traits<element_type,char_type>::encode(std::get<Size-Pos>(tuple), encoder, proto, ec);
-                if (ec) {return;}
+                if (JSONCONS_UNLIKELY(ec)) {return;}
                 next::encode(tuple, encoder, proto, ec);
             }
         };
@@ -237,9 +237,9 @@ namespace jsoncons {
         {
             using helper = jsoncons::detail::json_serialize_tuple_helper<size, size, Json, std::tuple<E...>>;
             encoder.begin_array(size,semantic_tag::none,ser_context(),ec);
-            if (ec) {return;}
+            if (JSONCONS_UNLIKELY(ec)) {return;}
             helper::encode(val, encoder, proto, ec);
-            if (ec) {return;}
+            if (JSONCONS_UNLIKELY(ec)) {return;}
             encoder.end_array(ser_context(),ec);
         }
     };
@@ -261,11 +261,11 @@ namespace jsoncons {
                            std::error_code& ec)
         {
             encoder.begin_array(val.size(),semantic_tag::none,ser_context(),ec);
-            if (ec) {return;}
+            if (JSONCONS_UNLIKELY(ec)) {return;}
             for (auto it = std::begin(val); it != std::end(val); ++it)
             {
                 encode_traits<value_type,CharT>::encode(*it, encoder, proto, ec);
-                if (ec) {return;}
+                if (JSONCONS_UNLIKELY(ec)) {return;}
             }
             encoder.end_array(ser_context(), ec);
         }
@@ -304,11 +304,11 @@ namespace jsoncons {
                            std::error_code& ec)
         {
             encoder.begin_array(val.size(),semantic_tag::none,ser_context(),ec);
-            if (ec) {return;}
+            if (JSONCONS_UNLIKELY(ec)) {return;}
             for (auto it = std::begin(val); it != std::end(val); ++it)
             {
                 encode_traits<value_type,CharT>::encode(*it, encoder, proto, ec);
-                if (ec) {return;}
+                if (JSONCONS_UNLIKELY(ec)) {return;}
             }
             encoder.end_array(ser_context(),ec);
         }
@@ -334,15 +334,15 @@ namespace jsoncons {
                            std::error_code& ec)
         {
             encoder.begin_object(val.size(), semantic_tag::none, ser_context(), ec);
-            if (ec) {return;}
+            if (JSONCONS_UNLIKELY(ec)) {return;}
             for (auto it = std::begin(val); it != std::end(val); ++it)
             {
                 encoder.key((*it).first);
                 encode_traits<mapped_type,CharT>::encode((*it).second, encoder, proto, ec);
-                if (ec) {return;}
+                if (JSONCONS_UNLIKELY(ec)) {return;}
             }
             encoder.end_object(ser_context(), ec);
-            if (ec) {return;}
+            if (JSONCONS_UNLIKELY(ec)) {return;}
         }
     };
 
@@ -364,17 +364,17 @@ namespace jsoncons {
                            std::error_code& ec)
         {
             encoder.begin_object(val.size(), semantic_tag::none, ser_context(), ec);
-            if (ec) {return;}
+            if (JSONCONS_UNLIKELY(ec)) {return;}
             for (auto it = std::begin(val); it != std::end(val); ++it)
             {
                 std::basic_string<typename Json::char_type> s;
                 jsoncons::detail::from_integer((*it).first,s);
                 encoder.key(s);
                 encode_traits<mapped_type,CharT>::encode((*it).second, encoder, proto, ec);
-                if (ec) {return;}
+                if (JSONCONS_UNLIKELY(ec)) {return;}
             }
             encoder.end_object(ser_context(), ec);
-            if (ec) {return;}
+            if (JSONCONS_UNLIKELY(ec)) {return;}
         }
     };
 
