@@ -226,16 +226,16 @@ namespace detail {
         class encoding_context
         {
             container_type type_;
-            std::size_t count_{0};
             line_split_kind line_splits_;
             bool indent_before_;
             bool new_line_after_;
             std::size_t begin_pos_{0};
             std::size_t data_pos_{0};
+            std::size_t count_{0};
         public:
             encoding_context(container_type type, line_split_kind split_lines, bool indent_once,
                              std::size_t begin_pos, std::size_t data_pos) noexcept
-               : type_(type), count_(0), line_splits_(split_lines), indent_before_(indent_once), new_line_after_(false),
+               : type_(type), line_splits_(split_lines), indent_before_(indent_once), new_line_after_(false),
                  begin_pos_(begin_pos), data_pos_(data_pos)
             {
             }
@@ -319,15 +319,15 @@ namespace detail {
         jsoncons::detail::write_double fp_;
 
         std::vector<encoding_context,encoding_context_allocator_type> stack_;
-        int indent_amount_;
-        std::size_t column_;
+        int indent_amount_{0};
+        std::size_t column_{0};
         std::basic_string<CharT> colon_str_;
         std::basic_string<CharT> comma_str_;
         std::basic_string<CharT> open_object_brace_str_;
         std::basic_string<CharT> close_object_brace_str_;
         std::basic_string<CharT> open_array_bracket_str_;
         std::basic_string<CharT> close_array_bracket_str_;
-        int nesting_depth_;
+        int nesting_depth_{0};
     public:
 
         // Noncopyable and nonmoveable
@@ -346,10 +346,7 @@ namespace detail {
            : sink_(std::forward<Sink>(sink)), 
              options_(options),
              fp_(options.float_format(), options.precision()),
-             stack_(alloc),
-             indent_amount_(0), 
-             column_(0),
-             nesting_depth_(0)
+             stack_(alloc)
         {
             switch (options.spaces_around_colon())
             {
@@ -438,12 +435,12 @@ namespace detail {
             sink_.flush();
         }
 
-        bool visit_begin_object(semantic_tag, const ser_context&, std::error_code& ec) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_begin_object(semantic_tag, const ser_context&, std::error_code& ec) final
         {
             if (JSONCONS_UNLIKELY(++nesting_depth_ > options_.max_nesting_depth()))
             {
                 ec = json_errc::max_nesting_depth_exceeded;
-                return false;
+                JSONCONS_VISITOR_RETURN;
             } 
             if (!stack_.empty() && stack_.back().is_array() && stack_.back().count() > 0)
             {
@@ -480,6 +477,11 @@ namespace detail {
                                 //stack_.back().new_line_after(true);
                                 new_line();
                             }
+                            else
+                            {
+                                stack_.back().new_line_after(true);
+                                new_line();
+                            }
                             break;
                         case line_split_kind::new_line:
                             stack_.back().new_line_after(true);
@@ -503,10 +505,10 @@ namespace detail {
             
             sink_.append(open_object_brace_str_.data(), open_object_brace_str_.length());
             column_ += open_object_brace_str_.length();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_end_object(const ser_context&, std::error_code&) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_end_object(const ser_context&, std::error_code&) final
         {
             JSONCONS_ASSERT(!stack_.empty());
             --nesting_depth_;
@@ -521,15 +523,15 @@ namespace detail {
             column_ += close_object_brace_str_.length();
 
             end_value();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_begin_array(semantic_tag, const ser_context&, std::error_code& ec) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_begin_array(semantic_tag, const ser_context&, std::error_code& ec) final
         {
             if (JSONCONS_UNLIKELY(++nesting_depth_ > options_.max_nesting_depth()))
             {
                 ec = json_errc::max_nesting_depth_exceeded;
-                return false;
+                JSONCONS_VISITOR_RETURN;
             } 
             if (!stack_.empty() && stack_.back().is_array() && stack_.back().count() > 0)
             {
@@ -595,10 +597,10 @@ namespace detail {
             indent();
             sink_.append(open_array_bracket_str_.data(), open_array_bracket_str_.length());
             column_ += open_array_bracket_str_.length();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_end_array(const ser_context&, std::error_code&) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_end_array(const ser_context&, std::error_code&) final
         {
             JSONCONS_ASSERT(!stack_.empty());
             --nesting_depth_;
@@ -612,10 +614,10 @@ namespace detail {
             sink_.append(close_array_bracket_str_.data(), close_array_bracket_str_.length());
             column_ += close_array_bracket_str_.length();
             end_value();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_key(const string_view_type& name, const ser_context&, std::error_code&) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_key(const string_view_type& name, const ser_context&, std::error_code&) final
         {
             JSONCONS_ASSERT(!stack_.empty());
             if (stack_.back().count() > 0)
@@ -644,10 +646,10 @@ namespace detail {
             sink_.push_back('\"');
             sink_.append(colon_str_.data(),colon_str_.length());
             column_ += (length+2+colon_str_.length());
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_null(semantic_tag, const ser_context&, std::error_code&) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_null(semantic_tag, const ser_context&, std::error_code&) final
         {
             if (!stack_.empty()) 
             {
@@ -665,10 +667,10 @@ namespace detail {
             column_ += null_constant().size();
 
             end_value();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_string(const string_view_type& sv, semantic_tag tag, const ser_context& context, std::error_code& ec) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_string(const string_view_type& sv, semantic_tag tag, const ser_context& context, std::error_code& ec) final
         {
             if (!stack_.empty()) 
             {
@@ -685,10 +687,10 @@ namespace detail {
             write_string(sv, tag, context, ec);
 
             end_value();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool write_string(const string_view_type& sv, semantic_tag tag, const ser_context&, std::error_code&) 
+        void write_string(const string_view_type& sv, semantic_tag tag, const ser_context&, std::error_code&) 
         {
             switch (tag)
             {
@@ -701,10 +703,10 @@ namespace detail {
                     if (options_.bignum_format() == bignum_format_kind::raw)
                     {
                         write_bigint_value(sv);
-                break;
-            }
-            JSONCONS_FALLTHROUGH;
-        }
+                        break;
+                    }
+                    JSONCONS_FALLTHROUGH;
+                }
                 default:
                 {
                     sink_.push_back('\"');
@@ -714,11 +716,9 @@ namespace detail {
                     break;
                 }
             }
-
-            return true;
         }
 
-        bool visit_byte_string(const byte_string_view& b, 
+        JSONCONS_VISITOR_RETURN_TYPE visit_byte_string(const byte_string_view& b, 
                                   semantic_tag tag,
                                   const ser_context&,
                                   std::error_code&) final
@@ -788,10 +788,10 @@ namespace detail {
             }
 
             end_value();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_double(double value, 
+        JSONCONS_VISITOR_RETURN_TYPE visit_double(double value, 
                              semantic_tag,
                              const ser_context& context,
                              std::error_code& ec) final
@@ -869,10 +869,10 @@ namespace detail {
             }
 
             end_value();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_int64(int64_t value, 
+        JSONCONS_VISITOR_RETURN_TYPE visit_int64(int64_t value, 
                             semantic_tag,
                             const ser_context&,
                             std::error_code&) final
@@ -891,10 +891,10 @@ namespace detail {
             std::size_t length = jsoncons::detail::from_integer(value, sink_);
             column_ += length;
             end_value();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_uint64(uint64_t value, 
+        JSONCONS_VISITOR_RETURN_TYPE visit_uint64(uint64_t value, 
                              semantic_tag, 
                              const ser_context&,
                              std::error_code&) final
@@ -913,10 +913,10 @@ namespace detail {
             std::size_t length = jsoncons::detail::from_integer(value, sink_);
             column_ += length;
             end_value();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_bool(bool value, semantic_tag, const ser_context&, std::error_code&) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_bool(bool value, semantic_tag, const ser_context&, std::error_code&) final
         {
             if (!stack_.empty()) 
             {
@@ -942,7 +942,7 @@ namespace detail {
             }
 
             end_value();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
         void begin_scalar_value()
@@ -1105,10 +1105,10 @@ namespace detail {
         class encoding_context
         {
             container_type type_;
-            std::size_t count_;
+            std::size_t count_{0};
         public:
             encoding_context(container_type type) noexcept
-               : type_(type), count_(0)
+               : type_(type)
             {
             }
 
@@ -1190,12 +1190,12 @@ namespace detail {
             sink_.flush();
         }
 
-        bool visit_begin_object(semantic_tag, const ser_context&, std::error_code& ec) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_begin_object(semantic_tag, const ser_context&, std::error_code& ec) final
         {
             if (JSONCONS_UNLIKELY(++nesting_depth_ > options_.max_nesting_depth()))
             {
                 ec = json_errc::max_nesting_depth_exceeded;
-                return false;
+                JSONCONS_VISITOR_RETURN;
             } 
             if (!stack_.empty() && stack_.back().is_array() && stack_.back().count() > 0)
             {
@@ -1204,10 +1204,10 @@ namespace detail {
 
             stack_.emplace_back(container_type::object);
             sink_.push_back('{');
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_end_object(const ser_context&, std::error_code&) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_end_object(const ser_context&, std::error_code&) final
         {
             JSONCONS_ASSERT(!stack_.empty());
             --nesting_depth_;
@@ -1219,16 +1219,16 @@ namespace detail {
             {
                 stack_.back().increment_count();
             }
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
 
-        bool visit_begin_array(semantic_tag, const ser_context&, std::error_code& ec) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_begin_array(semantic_tag, const ser_context&, std::error_code& ec) final
         {
             if (JSONCONS_UNLIKELY(++nesting_depth_ > options_.max_nesting_depth()))
             {
                 ec = json_errc::max_nesting_depth_exceeded;
-                return false;
+                JSONCONS_VISITOR_RETURN;
             } 
             if (!stack_.empty() && stack_.back().is_array() && stack_.back().count() > 0)
             {
@@ -1236,10 +1236,10 @@ namespace detail {
             }
             stack_.emplace_back(container_type::array);
             sink_.push_back('[');
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_end_array(const ser_context&, std::error_code&) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_end_array(const ser_context&, std::error_code&) final
         {
             JSONCONS_ASSERT(!stack_.empty());
             --nesting_depth_;
@@ -1250,10 +1250,10 @@ namespace detail {
             {
                 stack_.back().increment_count();
             }
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_key(const string_view_type& name, const ser_context&, std::error_code&) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_key(const string_view_type& name, const ser_context&, std::error_code&) final
         {
             if (!stack_.empty() && stack_.back().count() > 0)
             {
@@ -1264,10 +1264,10 @@ namespace detail {
             jsoncons::detail::escape_string(name.data(), name.length(),options_.escape_all_non_ascii(),options_.escape_solidus(),sink_);
             sink_.push_back('\"');
             sink_.push_back(':');
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_null(semantic_tag, const ser_context&, std::error_code&) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_null(semantic_tag, const ser_context&, std::error_code&) final
         {
             if (!stack_.empty() && stack_.back().is_array() && stack_.back().count() > 0)
             {
@@ -1280,7 +1280,7 @@ namespace detail {
             {
                 stack_.back().increment_count();
             }
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
         void write_bigint_value(const string_view_type& sv)
@@ -1344,7 +1344,7 @@ namespace detail {
             }
         }
 
-        bool visit_string(const string_view_type& sv, semantic_tag tag, const ser_context&, std::error_code&) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_string(const string_view_type& sv, semantic_tag tag, const ser_context&, std::error_code&) final
         {
             if (!stack_.empty() && stack_.back().is_array() && stack_.back().count() > 0)
             {
@@ -1363,9 +1363,9 @@ namespace detail {
                     {
                         write_bigint_value(sv);
                         break;
-            }
-            JSONCONS_FALLTHROUGH;
-        }
+                    }
+                    JSONCONS_FALLTHROUGH;
+                }
                 default:
                 {
                     sink_.push_back('\"');
@@ -1379,10 +1379,10 @@ namespace detail {
             {
                 stack_.back().increment_count();
             }
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool write_string(const string_view_type& sv, semantic_tag tag, const ser_context&, std::error_code&) 
+        void write_string(const string_view_type& sv, semantic_tag tag, const ser_context&, std::error_code&) 
         {
             switch (tag)
             {
@@ -1396,9 +1396,9 @@ namespace detail {
                     {
                         write_bigint_value(sv);
                         break;
-            }
-            JSONCONS_FALLTHROUGH;
-        }
+                    }
+                    JSONCONS_FALLTHROUGH;
+                }
                 default:
                 {
                     sink_.push_back('\"');
@@ -1407,10 +1407,9 @@ namespace detail {
                     break;
                 }
             }
-            return true;
         }
 
-        bool visit_byte_string(const byte_string_view& b, 
+        JSONCONS_VISITOR_RETURN_TYPE visit_byte_string(const byte_string_view& b, 
                                   semantic_tag tag,
                                   const ser_context&,
                                   std::error_code&) final
@@ -1473,10 +1472,10 @@ namespace detail {
             {
                 stack_.back().increment_count();
             }
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_double(double value, 
+        JSONCONS_VISITOR_RETURN_TYPE visit_double(double value, 
                              semantic_tag,
                              const ser_context& context,
                              std::error_code& ec) final
@@ -1543,10 +1542,10 @@ namespace detail {
             {
                 stack_.back().increment_count();
             }
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_int64(int64_t value, 
+        JSONCONS_VISITOR_RETURN_TYPE visit_int64(int64_t value, 
                             semantic_tag,
                             const ser_context&,
                             std::error_code&) final
@@ -1560,10 +1559,10 @@ namespace detail {
             {
                 stack_.back().increment_count();
             }
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_uint64(uint64_t value, 
+        JSONCONS_VISITOR_RETURN_TYPE visit_uint64(uint64_t value, 
                              semantic_tag, 
                              const ser_context&,
                              std::error_code&) final
@@ -1577,10 +1576,10 @@ namespace detail {
             {
                 stack_.back().increment_count();
             }
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_bool(bool value, semantic_tag, const ser_context&, std::error_code&) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_bool(bool value, semantic_tag, const ser_context&, std::error_code&) final
         {
             if (!stack_.empty() && stack_.back().is_array() && stack_.back().count() > 0)
             {
@@ -1600,7 +1599,7 @@ namespace detail {
             {
                 stack_.back().increment_count();
             }
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
     };
 
