@@ -47,23 +47,16 @@ namespace jsoncons {
         using char_type = CharT;
         using typename super_type::string_view_type;
     private:
-        std::function<bool(const basic_staj_event<CharT>&, const ser_context&)> pred_;
         basic_staj_event<CharT> event_;
 
         item_event_reader_state state_;
         typed_array_view data_;
         jsoncons::span<const size_t> shape_;
-        std::size_t index_;
+        std::size_t index_{0};
     public:
         basic_item_event_receiver()
-            : pred_(accept), event_(staj_event_type::null_value),
-              state_(), data_(), shape_(), index_(0)
-        {
-        }
-
-        basic_item_event_receiver(std::function<bool(const basic_staj_event<CharT>&, const ser_context&)> pred)
-            : pred_(pred), event_(staj_event_type::null_value),
-              state_(), data_(), shape_(), index_(0)
+            : event_(staj_event_type::null_value),
+              state_(), data_(), shape_()
         {
         }
 
@@ -214,83 +207,103 @@ namespace jsoncons {
             }
         }
 
-        bool dump(basic_item_event_visitor<CharT>& visitor, const ser_context& context, std::error_code& ec)
+        void dump(basic_item_event_visitor<CharT>& visitor, const ser_context& context, std::error_code& ec)
         {
-            bool more = true;
             if (is_typed_array())
             {
                 if (index_ != 0)
                 {
-                    more = event().send_value_event(visitor, context, ec);
-                    while (more && is_typed_array())
+                    event().send_value_event(visitor, context, ec);
+                    const std::size_t len = data_.size();
+                    switch (data_.type())
                     {
-                        if (index_ < data_.size())
+                        case typed_array_type::uint8_value:
                         {
-                            switch (data_.type())
+                            for (auto i = index_; i < len; ++i) 
                             {
-                                case typed_array_type::uint8_value:
-                                {
-                                    more = visitor.uint64_value(data_.data(uint8_array_arg)[index_]);
-                                    break;
-                                }
-                                case typed_array_type::uint16_value:
-                                {
-                                    more = visitor.uint64_value(data_.data(uint16_array_arg)[index_]);
-                                    break;
-                                }
-                                case typed_array_type::uint32_value:
-                                {
-                                    more = visitor.uint64_value(data_.data(uint32_array_arg)[index_]);
-                                    break;
-                                }
-                                case typed_array_type::uint64_value:
-                                {
-                                    more = visitor.uint64_value(data_.data(uint64_array_arg)[index_]);
-                                    break;
-                                }
-                                case typed_array_type::int8_value:
-                                {
-                                    more = visitor.int64_value(data_.data(int8_array_arg)[index_]);
-                                    break;
-                                }
-                                case typed_array_type::int16_value:
-                                {
-                                    more = visitor.int64_value(data_.data(int16_array_arg)[index_]);
-                                    break;
-                                }
-                                case typed_array_type::int32_value:
-                                {
-                                    more = visitor.int64_value(data_.data(int32_array_arg)[index_]);
-                                    break;
-                                }
-                                case typed_array_type::int64_value:
-                                {
-                                    more = visitor.int64_value(data_.data(int64_array_arg)[index_]);
-                                    break;
-                                }
-                                case typed_array_type::float_value:
-                                {
-                                    more = visitor.double_value(data_.data(float_array_arg)[index_]);
-                                    break;
-                                }
-                                case typed_array_type::double_value:
-                                {
-                                    more = visitor.double_value(data_.data(double_array_arg)[index_]);
-                                    break;
-                                }
-                                default:
-                                    break;
+                                visitor.uint64_value(data_.data(uint8_array_arg)[i]);
                             }
-                            ++index_;
+                            break;
                         }
-                        else
+                        case typed_array_type::uint16_value:
                         {
-                            more = visitor.end_array();
-                            state_ = item_event_reader_state();
-                            data_ = typed_array_view();
-                            index_ = 0;
+                            for (auto i = index_; i < len; ++i) 
+                            {
+                                visitor.uint64_value(data_.data(uint16_array_arg)[i]);
+                            }
+                            break;
                         }
+                        case typed_array_type::uint32_value:
+                        {
+                            for (auto i = index_; i < len; ++i) 
+                            {
+                                visitor.uint64_value(data_.data(uint32_array_arg)[i]);
+                            }
+                            break;
+                        }
+                        case typed_array_type::uint64_value:
+                        {
+                            for (auto i = index_; i < len; ++i) 
+                            {
+                                visitor.uint64_value(data_.data(uint64_array_arg)[i]);
+                            }
+                            break;
+                        }
+                        case typed_array_type::int8_value:
+                        {
+                            for (auto i = index_; i < len; ++i) 
+                            {
+                                visitor.int64_value(data_.data(int8_array_arg)[i]);
+                            }
+                            break;
+                        }
+                        case typed_array_type::int16_value:
+                        {
+                            for (auto i = index_; i < len; ++i) 
+                            {
+                                visitor.int64_value(data_.data(int16_array_arg)[i]);
+                            }
+                            break;
+                        }
+                        case typed_array_type::int32_value:
+                        {
+                            for (auto i = index_; i < len; ++i) 
+                            {
+                                visitor.int64_value(data_.data(int32_array_arg)[i]);
+                            }
+                            break;
+                        }
+                        case typed_array_type::int64_value:
+                        {
+                            for (auto i = index_; i < len; ++i) 
+                            {
+                                visitor.int64_value(data_.data(int64_array_arg)[i]);
+                            }
+                            break;
+                        }
+                        case typed_array_type::float_value:
+                        {
+                            for (auto i = index_; i < len; ++i) 
+                            {
+                                visitor.double_value(data_.data(float_array_arg)[i]);
+                            }
+                            break;
+                        }
+                        case typed_array_type::double_value:
+                        {
+                            for (auto i = index_; i < len; ++i) 
+                            {
+                                visitor.double_value(data_.data(double_array_arg)[i]);
+                            }
+                            break;
+                        }
+                        default:
+                            break;
                     }
+
+                    state_ = item_event_reader_state();
+                    data_ = typed_array_view();
+                    index_ = 0;
                 }
                 else
                 {
@@ -298,52 +311,52 @@ namespace jsoncons {
                     {
                         case typed_array_type::uint8_value:
                         {
-                            more = visitor.typed_array(data_.data(uint8_array_arg));
+                            visitor.typed_array(data_.data(uint8_array_arg));
                             break;
                         }
                         case typed_array_type::uint16_value:
                         {
-                            more = visitor.typed_array(data_.data(uint16_array_arg));
+                            visitor.typed_array(data_.data(uint16_array_arg));
                             break;
                         }
                         case typed_array_type::uint32_value:
                         {
-                            more = visitor.typed_array(data_.data(uint32_array_arg));
+                            visitor.typed_array(data_.data(uint32_array_arg));
                             break;
                         }
                         case typed_array_type::uint64_value:
                         {
-                            more = visitor.typed_array(data_.data(uint64_array_arg));
+                            visitor.typed_array(data_.data(uint64_array_arg));
                             break;
                         }
                         case typed_array_type::int8_value:
                         {
-                            more = visitor.typed_array(data_.data(int8_array_arg));
+                            visitor.typed_array(data_.data(int8_array_arg));
                             break;
                         }
                         case typed_array_type::int16_value:
                         {
-                            more = visitor.typed_array(data_.data(int16_array_arg));
+                            visitor.typed_array(data_.data(int16_array_arg));
                             break;
                         }
                         case typed_array_type::int32_value:
                         {
-                            more = visitor.typed_array(data_.data(int32_array_arg));
+                            visitor.typed_array(data_.data(int32_array_arg));
                             break;
                         }
                         case typed_array_type::int64_value:
                         {
-                            more = visitor.typed_array(data_.data(int64_array_arg));
+                            visitor.typed_array(data_.data(int64_array_arg));
                             break;
                         }
                         case typed_array_type::float_value:
                         {
-                            more = visitor.typed_array(data_.data(float_array_arg));
+                            visitor.typed_array(data_.data(float_array_arg));
                             break;
                         }
                         case typed_array_type::double_value:
                         {
-                            more = visitor.typed_array(data_.data(double_array_arg));
+                            visitor.typed_array(data_.data(double_array_arg));
                             break;
                         }
                         default:
@@ -356,268 +369,276 @@ namespace jsoncons {
             }
             else
             {
-                more = event().send_value_event(visitor, context, ec);
+                event().send_value_event(visitor, context, ec);
             }
-            return more;
         }
 
     private:
-        static constexpr bool accept(const basic_staj_event<CharT>&, const ser_context&) 
-        {
-            return true;
-        }
 
-        bool visit_begin_object(semantic_tag tag, const ser_context& context, std::error_code&) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_begin_object(semantic_tag tag, const ser_context&, std::error_code&) override
         {
             event_ = basic_staj_event<CharT>(staj_event_type::begin_object, tag);
-            return !pred_(event_, context);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_begin_object(std::size_t length, semantic_tag tag, const ser_context& context, std::error_code&) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_begin_object(std::size_t length, semantic_tag tag, const ser_context&, std::error_code&) override
         {
             event_ = basic_staj_event<CharT>(staj_event_type::begin_object, length, tag);
-            return !pred_(event_, context);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_end_object(const ser_context& context, std::error_code&) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_end_object(const ser_context&, std::error_code&) override
         {
             event_ = basic_staj_event<CharT>(staj_event_type::end_object);
-            return !pred_(event_, context);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_begin_array(semantic_tag tag, const ser_context& context, std::error_code&) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_begin_array(semantic_tag tag, const ser_context&, std::error_code&) override
         {
             event_ = basic_staj_event<CharT>(staj_event_type::begin_array, tag);
-            return !pred_(event_, context);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_begin_array(std::size_t length, semantic_tag tag, const ser_context& context, std::error_code&) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_begin_array(std::size_t length, semantic_tag tag, const ser_context&, std::error_code&) override
         {
             event_ = basic_staj_event<CharT>(staj_event_type::begin_array, length, tag);
-            return !pred_(event_, context);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_end_array(const ser_context& context, std::error_code&) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_end_array(const ser_context&, std::error_code&) override
         {
             event_ = basic_staj_event<CharT>(staj_event_type::end_array);
-            return !pred_(event_, context);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_null(semantic_tag tag, const ser_context& context, std::error_code&) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_null(semantic_tag tag, const ser_context&, std::error_code&) override
         {
             event_ = basic_staj_event<CharT>(staj_event_type::null_value, tag);
-            return !pred_(event_, context);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_bool(bool value, semantic_tag tag, const ser_context& context, std::error_code&) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_bool(bool value, semantic_tag tag, const ser_context&, std::error_code&) override
         {
             event_ = basic_staj_event<CharT>(value, tag);
-            return !pred_(event_, context);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_string(const string_view_type& s, semantic_tag tag, const ser_context& context, std::error_code&) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_string(const string_view_type& s, semantic_tag tag, const ser_context&, std::error_code&) override
         {
             event_ = basic_staj_event<CharT>(s, staj_event_type::string_value, tag);
-            return !pred_(event_, context);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_byte_string(const byte_string_view& s, 
-                               semantic_tag tag,
-                               const ser_context& context,
-                               std::error_code&) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_byte_string(const byte_string_view& s, 
+            semantic_tag tag,
+            const ser_context&,
+            std::error_code&) override
         {
             event_ = basic_staj_event<CharT>(s, staj_event_type::byte_string_value, tag);
-            return !pred_(event_, context);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_byte_string(const byte_string_view& s, 
-                               uint64_t ext_tag,
-                               const ser_context& context,
-                               std::error_code&) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_byte_string(const byte_string_view& s, 
+            uint64_t ext_tag,
+            const ser_context&,
+            std::error_code&) override
         {
             event_ = basic_staj_event<CharT>(s, staj_event_type::byte_string_value, ext_tag);
-            return !pred_(event_, context);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_uint64(uint64_t value, 
-                             semantic_tag tag, 
-                             const ser_context& context,
-                             std::error_code&) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_uint64(uint64_t value, 
+            semantic_tag tag, 
+            const ser_context&,
+            std::error_code&) override
         {
             event_ = basic_staj_event<CharT>(value, tag);
-            return !pred_(event_, context);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_int64(int64_t value, 
-                      semantic_tag tag,
-                      const ser_context& context,
-                      std::error_code&) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_int64(int64_t value, 
+            semantic_tag tag,
+            const ser_context&,
+            std::error_code&) override
         {
             event_ = basic_staj_event<CharT>(value, tag);
-            return !pred_(event_, context);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_half(uint16_t value, 
-                     semantic_tag tag,
-                     const ser_context& context,
-                     std::error_code&) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_half(uint16_t value, 
+            semantic_tag tag,
+            const ser_context&,
+            std::error_code&) override
         {
             event_ = basic_staj_event<CharT>(half_arg, value, tag);
-            return !pred_(event_, context);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_double(double value, 
-                       semantic_tag tag, 
-                       const ser_context& context,
-                       std::error_code&) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_double(double value, 
+            semantic_tag tag, 
+            const ser_context&,
+            std::error_code&) override
         {
             event_ = basic_staj_event<CharT>(value, tag);
-            return !pred_(event_, context);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_typed_array(const jsoncons::span<const uint8_t>& v, 
-                            semantic_tag tag,
-                            const ser_context& context,
-                            std::error_code& ec) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_typed_array(const jsoncons::span<const uint8_t>& v, 
+            semantic_tag tag,
+            const ser_context& context,
+            std::error_code& ec) override
         {
             state_ = item_event_reader_state::typed_array;
             data_ = typed_array_view(v.data(), v.size());
             index_ = 0;
-            return this->begin_array(tag, context, ec);
+            this->begin_array(tag, context, ec);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_typed_array(const jsoncons::span<const uint16_t>& data, 
-                            semantic_tag tag,
-                            const ser_context& context,
-                            std::error_code& ec) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_typed_array(const jsoncons::span<const uint16_t>& data, 
+            semantic_tag tag,
+            const ser_context& context,
+            std::error_code& ec) override
         {
             state_ = item_event_reader_state::typed_array;
             data_ = typed_array_view(data.data(), data.size());
             index_ = 0;
-            return this->begin_array(tag, context, ec);
+            this->begin_array(tag, context, ec);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_typed_array(const jsoncons::span<const uint32_t>& data, 
-                            semantic_tag tag,
-                            const ser_context& context,
-                            std::error_code& ec) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_typed_array(const jsoncons::span<const uint32_t>& data, 
+            semantic_tag tag,
+            const ser_context& context,
+            std::error_code& ec) override
         {
             state_ = item_event_reader_state::typed_array;
             data_ = typed_array_view(data.data(), data.size());
             index_ = 0;
-            return this->begin_array(tag, context, ec);
+            this->begin_array(tag, context, ec);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_typed_array(const jsoncons::span<const uint64_t>& data, 
-                            semantic_tag tag,
-                            const ser_context& context,
-                            std::error_code& ec) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_typed_array(const jsoncons::span<const uint64_t>& data, 
+            semantic_tag tag,
+            const ser_context& context,
+            std::error_code& ec) override
         {
             state_ = item_event_reader_state::typed_array;
             data_ = typed_array_view(data.data(), data.size());
             index_ = 0;
-            return this->begin_array(tag, context, ec);
+            this->begin_array(tag, context, ec);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_typed_array(const jsoncons::span<const int8_t>& data, 
-                            semantic_tag tag,
-                            const ser_context& context,
-                            std::error_code& ec) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_typed_array(const jsoncons::span<const int8_t>& data, 
+            semantic_tag tag,
+            const ser_context& context,
+            std::error_code& ec) override
         {
             state_ = item_event_reader_state::typed_array;
             data_ = typed_array_view(data.data(), data.size());
             index_ = 0;
-            return this->begin_array(tag, context, ec);
+            this->begin_array(tag, context, ec);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_typed_array(const jsoncons::span<const int16_t>& data, 
-                            semantic_tag tag,
-                            const ser_context& context,
-                            std::error_code& ec) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_typed_array(const jsoncons::span<const int16_t>& data, 
+            semantic_tag tag,
+            const ser_context& context,
+            std::error_code& ec) override
         {
             state_ = item_event_reader_state::typed_array;
             data_ = typed_array_view(data.data(), data.size());
             index_ = 0;
-            return this->begin_array(tag, context, ec);
+            this->begin_array(tag, context, ec);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_typed_array(const jsoncons::span<const int32_t>& data, 
-                            semantic_tag tag,
-                            const ser_context& context,
-                            std::error_code& ec) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_typed_array(const jsoncons::span<const int32_t>& data, 
+            semantic_tag tag,
+            const ser_context& context,
+            std::error_code& ec) override
         {
             state_ = item_event_reader_state::typed_array;
             data_ = typed_array_view(data.data(), data.size());
             index_ = 0;
-            return this->begin_array(tag, context, ec);
+            this->begin_array(tag, context, ec);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_typed_array(const jsoncons::span<const int64_t>& data, 
-                            semantic_tag tag,
-                            const ser_context& context,
-                            std::error_code& ec) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_typed_array(const jsoncons::span<const int64_t>& data, 
+            semantic_tag tag,
+            const ser_context& context,
+            std::error_code& ec) override
         {
             state_ = item_event_reader_state::typed_array;
             data_ = typed_array_view(data.data(), data.size());
             index_ = 0;
-            return this->begin_array(tag, context, ec);
+            this->begin_array(tag, context, ec);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_typed_array(half_arg_t, const jsoncons::span<const uint16_t>& data, 
-                            semantic_tag tag,
-                            const ser_context& context,
-                            std::error_code& ec) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_typed_array(half_arg_t, const jsoncons::span<const uint16_t>& data, 
+            semantic_tag tag,
+            const ser_context& context,
+            std::error_code& ec) override
         {
             state_ = item_event_reader_state::typed_array;
             data_ = typed_array_view(data.data(), data.size());
             index_ = 0;
-            return this->begin_array(tag, context, ec);
+            this->begin_array(tag, context, ec);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_typed_array(const jsoncons::span<const float>& data, 
-                            semantic_tag tag,
-                            const ser_context& context,
-                            std::error_code& ec) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_typed_array(const jsoncons::span<const float>& data, 
+            semantic_tag tag,
+            const ser_context& context,
+            std::error_code& ec) override
         {
             state_ = item_event_reader_state::typed_array;
             data_ = typed_array_view(data.data(), data.size());
             index_ = 0;
-            return this->begin_array(tag, context, ec);
+            this->begin_array(tag, context, ec);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_typed_array(const jsoncons::span<const double>& data, 
-                            semantic_tag tag,
-                            const ser_context& context,
-                            std::error_code& ec) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_typed_array(const jsoncons::span<const double>& data, 
+            semantic_tag tag,
+            const ser_context& context,
+            std::error_code& ec) override
         {
             state_ = item_event_reader_state::typed_array;
             data_ = typed_array_view(data.data(), data.size());
             index_ = 0;
-            return this->begin_array(tag, context, ec);
+            this->begin_array(tag, context, ec);
+            JSONCONS_VISITOR_RETURN;
         }
     /*
-        bool visit_typed_array(const jsoncons::span<const float128_type>&, 
+        JSONCONS_VISITOR_RETURN_TYPE visit_typed_array(const jsoncons::span<const float128_type>&, 
                             semantic_tag,
                             const ser_context&,
                             std::error_code&) override
         {
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
     */
-        bool visit_begin_multi_dim(const jsoncons::span<const size_t>& shape,
-                                semantic_tag tag,
-                                const ser_context& context, 
-                                std::error_code& ec) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_begin_multi_dim(const jsoncons::span<const size_t>& shape,
+            semantic_tag tag,
+            const ser_context& context, 
+            std::error_code& ec) override
         {
             state_ = item_event_reader_state::multi_dim;
             shape_ = shape;
-            return this->begin_array(2, tag, context, ec);
+            this->begin_array(2, tag, context, ec);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_end_multi_dim(const ser_context& context,
-                              std::error_code& ec) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_end_multi_dim(const ser_context& context,
+            std::error_code& ec) override
         {
-            return this->end_array(context, ec);
+            this->end_array(context, ec);
+            JSONCONS_VISITOR_RETURN;
         }
 
         void visit_flush() override
@@ -664,7 +685,7 @@ namespace jsoncons {
         std::function<bool(const basic_staj_event<CharT>&, const ser_context&)> pred_;
     public:
         basic_staj2_filter_view(basic_staj_event_reader<CharT>& cursor,
-                         std::function<bool(const basic_staj_event<CharT>&, const ser_context&)> pred)
+            std::function<bool(const basic_staj_event<CharT>&, const ser_context&)> pred)
             : cursor_(std::addressof(cursor)), pred_(pred)
         {
             while (!done() && !pred_(current(),context()))
