@@ -1,4 +1,4 @@
-// Copyright 2013-2024 Daniel Parker
+// Copyright 2013-2025 Daniel Parker
 // Distributed under the Boost license, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
@@ -16,14 +16,16 @@
 #ifndef JSONCONS_UTILITY_UNICODE_TRAITS_HPP
 #define JSONCONS_UTILITY_UNICODE_TRAITS_HPP
 
+#include <cstddef>
+#include <cstdint>
 #include <cstring>
 #include <iterator>
-#include <limits>
 #include <string>
 #include <system_error>
 #include <type_traits>
 
 #include <jsoncons/config/compiler_support.hpp>
+#include <jsoncons/config/jsoncons_config.hpp>
 #include <jsoncons/utility/extension_traits.hpp>
 
 namespace jsoncons { namespace unicode_traits {
