@@ -36,7 +36,7 @@ private:
     basic_cbor_parser<Source,Allocator> parser_;
     basic_staj_visitor<char_type> cursor_visitor_;
     basic_item_event_visitor_to_json_visitor<char_type,Allocator> cursor_handler_adaptor_;
-    bool eof_;
+    bool eof_{false};
 
 public:
     using string_view_type = string_view;
@@ -47,13 +47,12 @@ public:
 
     template <typename Sourceable>
     basic_cbor_cursor(Sourceable&& source,
-                      const cbor_decode_options& options = cbor_decode_options(),
-                      const Allocator& alloc = Allocator())
+        const cbor_decode_options& options = cbor_decode_options(),
+        const Allocator& alloc = Allocator())
         : parser_(std::forward<Sourceable>(source), options, alloc), 
-          cursor_visitor_(accept_all), 
-          cursor_handler_adaptor_(cursor_visitor_, alloc),
-          eof_(false)
+          cursor_handler_adaptor_(cursor_visitor_, alloc)
     {
+        parser_.cursor_mode(true);
         if (!done())
         {
             next();
@@ -64,35 +63,35 @@ public:
 
     template <typename Sourceable>
     basic_cbor_cursor(Sourceable&& source, 
-                      std::error_code& ec)
+        std::error_code& ec)
         : basic_cbor_cursor(std::allocator_arg, Allocator(),
-                            std::forward<Sourceable>(source), 
-                            cbor_decode_options(), 
-                            ec)
+              std::forward<Sourceable>(source), 
+              cbor_decode_options(), 
+              ec)
     {
     }
 
     template <typename Sourceable>
     basic_cbor_cursor(Sourceable&& source, 
-                      const cbor_decode_options& options,
-                      std::error_code& ec)
+        const cbor_decode_options& options,
+        std::error_code& ec)
         : basic_cbor_cursor(std::allocator_arg, Allocator(),
-                            std::forward<Sourceable>(source), 
-                            options, 
-                            ec)
+              std::forward<Sourceable>(source), 
+              options, 
+              ec)
     {
     }
 
     template <typename Sourceable>
     basic_cbor_cursor(std::allocator_arg_t, const Allocator& alloc, 
-                      Sourceable&& source,
-                      const cbor_decode_options& options,
-                      std::error_code& ec)
+        Sourceable&& source,
+        const cbor_decode_options& options,
+        std::error_code& ec)
        : parser_(std::forward<Sourceable>(source), options, alloc), 
-         cursor_visitor_(accept_all),
          cursor_handler_adaptor_(cursor_visitor_, alloc),
          eof_(false)
     {
+        parser_.cursor_mode(true);
         if (!done())
         {
             next(ec);
@@ -178,18 +177,43 @@ public:
     {
         std::error_code ec;
         read_to(visitor, ec);
-        if (ec)
+        if (JSONCONS_UNLIKELY(ec))
         {
             JSONCONS_THROW(ser_error(ec,parser_.line(),parser_.column()));
         }
     }
 
     void read_to(basic_json_visitor<char_type>& visitor,
-                 std::error_code& ec) override
+        std::error_code& ec) override
     {
-        if (cursor_visitor_.dump(visitor, *this, ec))
+        if (is_typed_array())
         {
+            cursor_visitor_.dump(visitor, *this, ec);
+        }
+        else if (is_begin_container(current().event_type()))
+        {
+            parser_.cursor_mode(false);
+            parser_.mark_level(parser_.level());
+            cursor_visitor_.event().send_json_event(visitor, *this, ec);
+            if (JSONCONS_UNLIKELY(ec))
+            {
+                return;
+            }
             read_next(visitor, ec);
+            parser_.cursor_mode(true);
+            parser_.mark_level(0);
+            if (current().event_type() == staj_event_type::begin_object)
+            {
+                cursor_visitor_.end_object(*this);
+            }
+            else
+            {
+                cursor_visitor_.end_array(*this);
+            }
+        }
+        else
+        {
+            cursor_visitor_.event().send_json_event(visitor, *this, ec);
         }
     }
 
@@ -197,7 +221,7 @@ public:
     {
         std::error_code ec;
         next(ec);
-        if (ec)
+        if (JSONCONS_UNLIKELY(ec))
         {
             JSONCONS_THROW(ser_error(ec,parser_.line(),parser_.column()));
         }
@@ -230,17 +254,12 @@ public:
 
     friend
     staj_filter_view operator|(basic_cbor_cursor& cursor, 
-                               std::function<bool(const staj_event&, const ser_context&)> pred)
+        std::function<bool(const staj_event&, const ser_context&)> pred)
     {
         return staj_filter_view(cursor, pred);
     }
 
 private:
-    static bool accept_all(const staj_event&, const ser_context&) 
-    {
-        return true;
-    }
-
     void read_next(std::error_code& ec)
     {
         if (cursor_visitor_.in_available())
@@ -253,7 +272,7 @@ private:
             while (!parser_.stopped())
             {
                 parser_.parse(cursor_handler_adaptor_, ec);
-                if (ec) {return;}
+                if (JSONCONS_UNLIKELY(ec)) {return;}
             }
         }
     }
@@ -283,7 +302,7 @@ private:
             while (!parser_.stopped())
             {
                 parser_.parse(cursor_handler_adaptor_, ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
