@@ -1,4 +1,4 @@
-/// Copyright 2013-2024 Daniel Parker
+/// Copyright 2013-2025 Daniel Parker
 // Distributed under the Boost license, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
@@ -7,10 +7,11 @@
 #ifndef JSONCONS_EXT_CBOR_CBOR_ERROR_HPP
 #define JSONCONS_EXT_CBOR_CBOR_ERROR_HPP
 
+#include <string>
 #include <system_error>
+#include <type_traits>
 
 #include <jsoncons/config/jsoncons_config.hpp>
-#include <jsoncons/json_exception.hpp> // jsoncons::ser_error
 
 namespace jsoncons { namespace cbor {
 
