@@ -124,23 +124,23 @@ class basic_cbor_parser : public ser_context
           item_tag,
           num_of_tags};
 
-    std::bitset<num_of_tags> other_tags_;
+    bool more_{true};
+    bool done_{false};
+    bool cursor_mode_{false};
+    int mark_level_{0};
+    uint64_t raw_tag_{0};
+    int nesting_depth_{0};
 
+    std::bitset<num_of_tags> other_tags_;
     allocator_type alloc_;
     Source source_;
     cbor_decode_options options_;
-
-    bool more_{true};
-    bool done_{false};
     string_type text_buffer_;
     byte_string_type bytes_buffer_;
-    uint64_t raw_tag_;
     std::vector<parse_state,parse_state_allocator_type> state_stack_;
     byte_string_type typed_array_;
     std::vector<std::size_t> shape_;
-    std::size_t index_; // TODO: Never used!
     std::vector<stringref_map,stringref_map_allocator_type> stringref_map_stack_;
-    int nesting_depth_;
 
     struct read_byte_string_from_buffer
     {
@@ -187,12 +187,9 @@ public:
          options_(options),
          text_buffer_(alloc),
          bytes_buffer_(alloc),
-         raw_tag_(0),
          state_stack_(alloc),
          typed_array_(alloc),
-         index_(0),
-         stringref_map_stack_(alloc),
-         nesting_depth_(0)
+         stringref_map_stack_(alloc)
     {
         state_stack_.emplace_back(parse_mode::root,0);
     }
@@ -230,6 +227,26 @@ public:
         reset();
     }
 
+    void cursor_mode(bool value)
+    {
+        cursor_mode_ = value;
+    }
+
+    int level() const
+    {
+        return static_cast<int>(state_stack_.size());
+    }
+
+    int mark_level() const 
+    {
+        return mark_level_;
+    }
+
+    void mark_level(int value)
+    {
+        mark_level_ = value;
+    }
+
     bool done() const
     {
         return done_;
@@ -290,7 +307,7 @@ public:
                 case parse_mode::indefinite_array:
                 {
                     auto c = source_.peek();
-                    if (c.eof)
+                    if (JSONCONS_UNLIKELY(c.eof))
                     {
                         ec = cbor_errc::unexpected_eof;
                         more_ = false;
@@ -330,7 +347,7 @@ public:
                 case parse_mode::indefinite_map_key:
                 {
                     auto c = source_.peek();
-                    if (c.eof)
+                    if (JSONCONS_UNLIKELY(c.eof))
                     {
                         ec = cbor_errc::unexpected_eof;
                         more_ = false;
@@ -376,12 +393,12 @@ private:
     void read_item(item_event_visitor& visitor, std::error_code& ec)
     {
         read_tags(ec);
-        if (!more_)
+        if (JSONCONS_UNLIKELY(ec))
         {
             return;
         }
         auto c = source_.peek();
-        if (c.eof)
+        if (JSONCONS_UNLIKELY(c.eof))
         {
             ec = cbor_errc::unexpected_eof;
             more_ = false;
@@ -395,7 +412,7 @@ private:
             case jsoncons::cbor::detail::cbor_major_type::unsigned_integer:
             {
                 uint64_t val = get_uint64_value(ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
@@ -421,7 +438,7 @@ private:
                         case jsoncons::cbor::detail::cbor_major_type::text_string:
                         {
                             handle_string(visitor, jsoncons::basic_string_view<char>(str.str.data(),str.str.length()),ec);
-                            if (ec)
+                            if (JSONCONS_UNLIKELY(ec))
                             {
                                 return;
                             }
@@ -431,7 +448,7 @@ private:
                         {
                             read_byte_string_from_buffer read(byte_string_view(str.bytes));
                             write_byte_string(read, visitor, ec);
-                            if (ec)
+                            if (JSONCONS_UNLIKELY(ec))
                             {
                                 return;
                             }
@@ -453,14 +470,15 @@ private:
                         }
                         other_tags_[item_tag] = false;
                     }
-                    more_ = visitor.uint64_value(val, tag, *this, ec);
+                    visitor.uint64_value(val, tag, *this, ec);
+                    more_ = !cursor_mode_;
                 }
                 break;
             }
             case jsoncons::cbor::detail::cbor_major_type::negative_integer:
             {
                 int64_t val = get_int64_value(ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
@@ -473,14 +491,15 @@ private:
                     }
                     other_tags_[item_tag] = false;
                 }
-                more_ = visitor.int64_value(val, tag, *this, ec);
+                visitor.int64_value(val, tag, *this, ec);
+                more_ = !cursor_mode_;
                 break;
             }
             case jsoncons::cbor::detail::cbor_major_type::byte_string:
             {
                 read_byte_string_from_source read(this);
                 write_byte_string(read, visitor, ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
@@ -491,7 +510,7 @@ private:
                 text_buffer_.clear();
 
                 read_text_string(text_buffer_, ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
@@ -503,7 +522,7 @@ private:
                     return;
                 }
                 handle_string(visitor, jsoncons::basic_string_view<char>(text_buffer_.data(),text_buffer_.length()),ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
@@ -519,36 +538,41 @@ private:
                 switch (info)
                 {
                     case 0x14:
-                        more_ = visitor.bool_value(false, semantic_tag::none, *this, ec);
+                        visitor.bool_value(false, semantic_tag::none, *this, ec);
+                        more_ = !cursor_mode_;
                         source_.ignore(1);
                         break;
                     case 0x15:
-                        more_ = visitor.bool_value(true, semantic_tag::none, *this, ec);
+                        visitor.bool_value(true, semantic_tag::none, *this, ec);
+                        more_ = !cursor_mode_;
                         source_.ignore(1);
                         break;
                     case 0x16:
-                        more_ = visitor.null_value(semantic_tag::none, *this, ec);
+                        visitor.null_value(semantic_tag::none, *this, ec);
+                        more_ = !cursor_mode_;
                         source_.ignore(1);
                         break;
                     case 0x17:
-                        more_ = visitor.null_value(semantic_tag::undefined, *this, ec);
+                        visitor.null_value(semantic_tag::undefined, *this, ec);
+                        more_ = !cursor_mode_;
                         source_.ignore(1);
                         break;
                     case 0x19: // Half-Precision Float (two-byte IEEE 754)
                     {
                         uint64_t val = get_uint64_value(ec);
-                        if (ec)
+                        if (JSONCONS_UNLIKELY(ec))
                         {
                             return;
                         }
-                        more_ = visitor.half_value(static_cast<uint16_t>(val), semantic_tag::none, *this, ec);
+                        visitor.half_value(static_cast<uint16_t>(val), semantic_tag::none, *this, ec);
+                        more_ = !cursor_mode_;
                         break;
                     }
                     case 0x1a: // Single-Precision Float (four-byte IEEE 754)
                     case 0x1b: // Double-Precision Float (eight-byte IEEE 754)
                     {
                         double val = get_double(ec);
-                        if (ec)
+                        if (JSONCONS_UNLIKELY(ec))
                         {
                             return;
                         }
@@ -561,7 +585,8 @@ private:
                             }
                             other_tags_[item_tag] = false;
                         }
-                        more_ = visitor.double_value(val, tag, *this, ec);
+                        visitor.double_value(val, tag, *this, ec);
+                        more_ = !cursor_mode_;
                         break;
                     }
                     default:
@@ -582,20 +607,22 @@ private:
                         case 0x04:
                             text_buffer_.clear();
                             read_decimal_fraction(text_buffer_, ec);
-                            if (ec)
+                            if (JSONCONS_UNLIKELY(ec))
                             {
                                 return;
                             }
-                            more_ = visitor.string_value(text_buffer_, semantic_tag::bigdec, *this, ec);
+                            visitor.string_value(text_buffer_, semantic_tag::bigdec, *this, ec);
+                            more_ = !cursor_mode_;
                             break;
                         case 0x05:
                             text_buffer_.clear();
                             read_bigfloat(text_buffer_, ec);
-                            if (ec)
+                            if (JSONCONS_UNLIKELY(ec))
                             {
                                 return;
                             }
-                            more_ = visitor.string_value(text_buffer_, semantic_tag::bigfloat, *this, ec);
+                            visitor.string_value(text_buffer_, semantic_tag::bigfloat, *this, ec);
+                            more_ = !cursor_mode_;
                             break;
                         case 40: // row major storage
                             produce_begin_multi_dim(visitor, semantic_tag::multi_dim_row_major, ec);
@@ -647,19 +674,21 @@ private:
             case jsoncons::cbor::detail::additional_info::indefinite_length:
             {
                 state_stack_.emplace_back(parse_mode::indefinite_array,0,pop_stringref_map_stack);
-                more_ = visitor.begin_array(tag, *this, ec);
+                visitor.begin_array(tag, *this, ec);
+                more_ = !cursor_mode_;
                 source_.ignore(1);
                 break;
             }
             default: // definite length
             {
                 std::size_t len = get_size(ec);
-                if (!more_)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
                 state_stack_.emplace_back(parse_mode::array,len,pop_stringref_map_stack);
-                more_ = visitor.begin_array(len, tag, *this, ec);
+                visitor.begin_array(len, tag, *this, ec);
+                more_ = !cursor_mode_;
                 break;
             }
         }
@@ -669,7 +698,12 @@ private:
     {
         --nesting_depth_;
 
-        more_ = visitor.end_array(*this, ec);
+        visitor.end_array(*this, ec);
+        more_ = !cursor_mode_;
+        if (level() == mark_level_)
+        {
+            more_ = false;
+        }
         if (state_stack_.back().pop_stringref_map_stack)
         {
             stringref_map_stack_.pop_back();
@@ -697,19 +731,21 @@ private:
             case jsoncons::cbor::detail::additional_info::indefinite_length: 
             {
                 state_stack_.emplace_back(parse_mode::indefinite_map_key,0,pop_stringref_map_stack);
-                more_ = visitor.begin_object(semantic_tag::none, *this, ec);
+                visitor.begin_object(semantic_tag::none, *this, ec);
+                more_ = !cursor_mode_;
                 source_.ignore(1);
                 break;
             }
             default: // definite_length
             {
                 std::size_t len = get_size(ec);
-                if (!more_)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
                 state_stack_.emplace_back(parse_mode::map_key,len,pop_stringref_map_stack);
-                more_ = visitor.begin_object(len, semantic_tag::none, *this, ec);
+                visitor.begin_object(len, semantic_tag::none, *this, ec);
+                more_ = !cursor_mode_;
                 break;
             }
         }
@@ -718,7 +754,12 @@ private:
     void end_object(item_event_visitor& visitor, std::error_code& ec)
     {
         --nesting_depth_;
-        more_ = visitor.end_object(*this, ec);
+        visitor.end_object(*this, ec);
+        if (level() == mark_level_)
+        {
+            more_ = false;
+        }
+        more_ = !cursor_mode_;
         if (state_stack_.back().pop_stringref_map_stack)
         {
             stringref_map_stack_.pop_back();
@@ -729,7 +770,7 @@ private:
     void read_text_string(string_type& str, std::error_code& ec)
     {
         auto c = source_.peek();
-        if (c.eof)
+        if (JSONCONS_UNLIKELY(c.eof))
         {
             ec = cbor_errc::unexpected_eof;
             more_ = false;
@@ -762,7 +803,7 @@ private:
     std::size_t get_size(std::error_code& ec)
     {
         uint64_t u = get_uint64_value(ec);
-        if (!more_)
+        if (JSONCONS_UNLIKELY(ec))
         {
             return 0;
         }
@@ -775,16 +816,14 @@ private:
         return len;
     }
 
-    bool read_byte_string(byte_string_type& v, std::error_code& ec)
+    void read_byte_string(byte_string_type& v, std::error_code& ec)
     {
-        bool more = true;
         v.clear();
         auto c = source_.peek();
-        if (c.eof)
+        if (JSONCONS_UNLIKELY(c.eof))
         {
             ec = cbor_errc::unexpected_eof;
-            more = false;
-            return more;
+            return;
         }
         jsoncons::cbor::detail::cbor_major_type major_type = get_major_type(c.value);
         uint8_t info = get_additional_information_value(c.value);
@@ -795,13 +834,12 @@ private:
         {
             case jsoncons::cbor::detail::additional_info::indefinite_length:
             {
-                auto func = [&v,&more](Source& source, std::size_t length, std::error_code& ec) -> bool
+                auto func = [&v](Source& source, std::size_t length, std::error_code& ec) -> bool
                 {
                     if (source_reader<Source>::read(source, v, length) != length)
                     {
                         ec = cbor_errc::unexpected_eof;
-                        more = false;
-                        return more;
+                        return false;
                     }
                     return true;
                 };
@@ -811,16 +849,14 @@ private:
             default:
             {
                 std::size_t length = get_size(ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
-                    more = false;
-                    return more;
+                    return;
                 }
                 if (source_reader<Source>::read(source_, v, length) != length)
                 {
                     ec = cbor_errc::unexpected_eof;
-                    more = false;
-                    return more;
+                    return;
                 }
                 if (!stringref_map_stack_.empty() &&
                     v.size() >= jsoncons::cbor::detail::min_length_for_stringref(stringref_map_stack_.back().size()))
@@ -831,7 +867,6 @@ private:
             }
 
         }
-        return more;
     }
 
     template <typename Function>
@@ -843,7 +878,7 @@ private:
         while (!done)
         {
             auto c = source_.peek();
-            if (c.eof)
+            if (JSONCONS_UNLIKELY(c.eof))
             {
                 ec = cbor_errc::unexpected_eof;
                 more_ = false;
@@ -880,12 +915,12 @@ private:
                 default: // definite length
                 {
                     std::size_t length = get_size(ec);
-                    if (!more_)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         return;
                     }
                     more_ = func(source_, length, ec);
-                    if (!more_)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         return;
                     }
@@ -1046,7 +1081,7 @@ private:
                 case jsoncons::cbor::detail::cbor_major_type::unsigned_integer:
                 {
                     uint64_t x = get_uint64_value(ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         return 0;
                     }
@@ -1118,7 +1153,7 @@ private:
     void read_decimal_fraction(string_type& result, std::error_code& ec)
     {
         std::size_t size = get_size(ec);
-        if (!more_)
+        if (JSONCONS_UNLIKELY(ec))
         {
             return;
         }
@@ -1130,7 +1165,7 @@ private:
         }
 
         auto c = source_.peek();
-        if (c.eof)
+        if (JSONCONS_UNLIKELY(c.eof))
         {
             ec = cbor_errc::unexpected_eof;
             more_ = false;
@@ -1142,7 +1177,7 @@ private:
             case jsoncons::cbor::detail::cbor_major_type::unsigned_integer:
             {
                 exponent = get_uint64_value(ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
@@ -1151,7 +1186,7 @@ private:
             case jsoncons::cbor::detail::cbor_major_type::negative_integer:
             {
                 exponent = get_int64_value(ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
@@ -1168,7 +1203,7 @@ private:
         string_type str(alloc_);
 
         c = source_.peek();
-        if (c.eof)
+        if (JSONCONS_UNLIKELY(c.eof))
         {
             ec = cbor_errc::unexpected_eof;
             more_ = false;
@@ -1180,7 +1215,7 @@ private:
             case jsoncons::cbor::detail::cbor_major_type::unsigned_integer:
             {
                 uint64_t val = get_uint64_value(ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
@@ -1190,7 +1225,7 @@ private:
             case jsoncons::cbor::detail::cbor_major_type::negative_integer:
             {
                 int64_t val = get_int64_value(ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
@@ -1208,7 +1243,7 @@ private:
                 }
                 uint8_t tag = get_additional_information_value(b);
                 c = source_.peek();
-                if (c.eof)
+                if (JSONCONS_UNLIKELY(c.eof))
                 {
                     ec = cbor_errc::unexpected_eof;
                     more_ = false;
@@ -1219,7 +1254,7 @@ private:
                 {
                     bytes_buffer_.clear();
                     read_byte_string(bytes_buffer_, ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
@@ -1277,7 +1312,7 @@ private:
     void read_bigfloat(string_type& str, std::error_code& ec)
     {
         std::size_t size = get_size(ec);
-        if (!more_)
+        if (JSONCONS_UNLIKELY(ec))
         {
             return;
         }
@@ -1289,7 +1324,7 @@ private:
         }
 
         auto c = source_.peek();
-        if (c.eof)
+        if (JSONCONS_UNLIKELY(c.eof))
         {
             ec = cbor_errc::unexpected_eof;
             more_ = false;
@@ -1301,7 +1336,7 @@ private:
             case jsoncons::cbor::detail::cbor_major_type::unsigned_integer:
             {
                 exponent = get_uint64_value(ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
@@ -1310,7 +1345,7 @@ private:
             case jsoncons::cbor::detail::cbor_major_type::negative_integer:
             {
                 exponent = get_int64_value(ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
@@ -1325,7 +1360,7 @@ private:
         }
 
         c = source_.peek();
-        if (c.eof)
+        if (JSONCONS_UNLIKELY(c.eof))
         {
             ec = cbor_errc::unexpected_eof;
             more_ = false;
@@ -1336,7 +1371,7 @@ private:
             case jsoncons::cbor::detail::cbor_major_type::unsigned_integer:
             {
                 uint64_t val = get_uint64_value(ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
@@ -1348,7 +1383,7 @@ private:
             case jsoncons::cbor::detail::cbor_major_type::negative_integer:
             {
                 int64_t val = get_int64_value(ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
@@ -1370,7 +1405,7 @@ private:
                 uint8_t tag = get_additional_information_value(b);
 
                 c = source_.peek();
-                if (c.eof)
+                if (JSONCONS_UNLIKELY(c.eof))
                 {
                     ec = cbor_errc::unexpected_eof;
                     more_ = false;
@@ -1380,9 +1415,10 @@ private:
                 if (get_major_type(c.value) == jsoncons::cbor::detail::cbor_major_type::byte_string)
                 {
                     bytes_buffer_.clear(); 
-                    more_ = read_byte_string(bytes_buffer_, ec);
-                    if (!more_)
+                    read_byte_string(bytes_buffer_, ec);
+                    if (JSONCONS_UNLIKELY(ec))
                     {
+                        more_ = false;
                         return;
                     }
                     if (tag == 2)
@@ -1441,7 +1477,7 @@ private:
     void read_tags(std::error_code& ec)
     {
         auto c = source_.peek();
-        if (c.eof)
+        if (JSONCONS_UNLIKELY(c.eof))
         {
             ec = cbor_errc::unexpected_eof;
             more_ = false;
@@ -1452,7 +1488,7 @@ private:
         while (major_type == jsoncons::cbor::detail::cbor_major_type::semantic_tag)
         {
             uint64_t val = get_uint64_value(ec);
-            if (!more_)
+            if (JSONCONS_UNLIKELY(ec))
             {
                 return;
             }
@@ -1470,7 +1506,7 @@ private:
                     break;
             }
             c = source_.peek();
-            if (c.eof)
+            if (JSONCONS_UNLIKELY(c.eof))
             {
                 ec = cbor_errc::unexpected_eof;
                 more_ = false;
@@ -1504,7 +1540,8 @@ private:
             }
             other_tags_[item_tag] = false;
         }
-        more_ = visitor.string_value(v, tag, *this, ec);
+        visitor.string_value(v, tag, *this, ec);
+        more_ = !cursor_mode_;
     }
 
     static jsoncons::endian get_typed_array_endianness(const uint8_t tag)
@@ -1531,7 +1568,7 @@ private:
                 {
                     bytes_buffer_.clear();
                     read(bytes_buffer_,ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
@@ -1539,14 +1576,15 @@ private:
                     bigint n = bigint::from_bytes_be(1, bytes_buffer_.data(), bytes_buffer_.size());
                     text_buffer_.clear();
                     n.write_string(text_buffer_);
-                    more_ = visitor.string_value(text_buffer_, semantic_tag::bigint, *this, ec);
+                    visitor.string_value(text_buffer_, semantic_tag::bigint, *this, ec);
+                    more_ = !cursor_mode_;
                     break;
                 }
                 case 0x3:
                 {
                     bytes_buffer_.clear();
                     read(bytes_buffer_,ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
@@ -1555,68 +1593,74 @@ private:
                     n = -1 - n;
                     text_buffer_.clear();
                     n.write_string(text_buffer_);
-                    more_ = visitor.string_value(text_buffer_, semantic_tag::bigint, *this, ec);
+                    visitor.string_value(text_buffer_, semantic_tag::bigint, *this, ec);
+                    more_ = !cursor_mode_;
                     break;
                 }
                 case 0x15:
                 {
                     read(bytes_buffer_,ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
                     }
-                    more_ = visitor.byte_string_value(bytes_buffer_, semantic_tag::base64url, *this, ec);
+                    visitor.byte_string_value(bytes_buffer_, semantic_tag::base64url, *this, ec);
+                    more_ = !cursor_mode_;
                     break;
                 }
                 case 0x16:
                 {
                     read(bytes_buffer_,ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
                     }
-                    more_ = visitor.byte_string_value(bytes_buffer_, semantic_tag::base64, *this, ec);
+                    visitor.byte_string_value(bytes_buffer_, semantic_tag::base64, *this, ec);
+                    more_ = !cursor_mode_;
                     break;
                 }
                 case 0x17:
                 {
                     read(bytes_buffer_,ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
                     }
-                    more_ = visitor.byte_string_value(bytes_buffer_, semantic_tag::base16, *this, ec);
+                    visitor.byte_string_value(bytes_buffer_, semantic_tag::base16, *this, ec);
+                    more_ = !cursor_mode_;
                     break;
                 }
                 case 0x40:
                 {
                     typed_array_.clear();
                     read(typed_array_,ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
                     }
                     uint8_t* data = reinterpret_cast<uint8_t*>(typed_array_.data());
                     std::size_t size = typed_array_.size();
-                    more_ = visitor.typed_array(jsoncons::span<const uint8_t>(data,size), semantic_tag::none, *this, ec);
+                    visitor.typed_array(jsoncons::span<const uint8_t>(data,size), semantic_tag::none, *this, ec);
+                    more_ = !cursor_mode_;
                     break;
                 }
                 case 0x44:
                 {
                     typed_array_.clear();
                     read(typed_array_,ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
                     }
                     uint8_t* data = reinterpret_cast<uint8_t*>(typed_array_.data());
                     std::size_t size = typed_array_.size();
-                    more_ = visitor.typed_array(jsoncons::span<const uint8_t>(data,size), semantic_tag::clamped, *this, ec);
+                    visitor.typed_array(jsoncons::span<const uint8_t>(data,size), semantic_tag::clamped, *this, ec);
+                    more_ = !cursor_mode_;
                     break;
                 }
                 case 0x41:
@@ -1624,7 +1668,7 @@ private:
                 {
                     typed_array_.clear();
                     read(typed_array_,ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
@@ -1643,7 +1687,8 @@ private:
                             data[i] = binary::byte_swap<uint16_t>(data[i]);
                         }
                     }
-                    more_ = visitor.typed_array(jsoncons::span<const uint16_t>(data,size), semantic_tag::none, *this, ec);
+                    visitor.typed_array(jsoncons::span<const uint16_t>(data,size), semantic_tag::none, *this, ec);
+                    more_ = !cursor_mode_;
                     break;
                 }
                 case 0x42:
@@ -1651,7 +1696,7 @@ private:
                 {
                     typed_array_.clear();
                     read(typed_array_,ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
@@ -1669,7 +1714,8 @@ private:
                             data[i] = binary::byte_swap<uint32_t>(data[i]);
                         }
                     }
-                    more_ = visitor.typed_array(jsoncons::span<const uint32_t>(data,size), semantic_tag::none, *this, ec);
+                    visitor.typed_array(jsoncons::span<const uint32_t>(data,size), semantic_tag::none, *this, ec);
+                    more_ = !cursor_mode_;
                     break;
                 }
                 case 0x43:
@@ -1677,7 +1723,7 @@ private:
                 {
                     typed_array_.clear();
                     read(typed_array_,ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
@@ -1695,21 +1741,23 @@ private:
                             data[i] = binary::byte_swap<uint64_t>(data[i]);
                         }
                     }
-                    more_ = visitor.typed_array(jsoncons::span<const uint64_t>(data,size), semantic_tag::none, *this, ec);
+                    visitor.typed_array(jsoncons::span<const uint64_t>(data,size), semantic_tag::none, *this, ec);
+                    more_ = !cursor_mode_;
                     break;
                 }
                 case 0x48:
                 {
                     typed_array_.clear();
                     read(typed_array_,ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
                     }
                     int8_t* data = reinterpret_cast<int8_t*>(typed_array_.data());
                     std::size_t size = typed_array_.size();
-                    more_ = visitor.typed_array(jsoncons::span<const int8_t>(data,size), semantic_tag::none, *this, ec);
+                    visitor.typed_array(jsoncons::span<const int8_t>(data,size), semantic_tag::none, *this, ec);
+                    more_ = !cursor_mode_;
                     break;
                 }
                 case 0x49:
@@ -1717,7 +1765,7 @@ private:
                 {
                     typed_array_.clear();
                     read(typed_array_,ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
@@ -1735,7 +1783,8 @@ private:
                             data[i] = binary::byte_swap<int16_t>(data[i]);
                         }
                     }
-                    more_ = visitor.typed_array(jsoncons::span<const int16_t>(data,size), semantic_tag::none, *this, ec);
+                    visitor.typed_array(jsoncons::span<const int16_t>(data,size), semantic_tag::none, *this, ec);
+                    more_ = !cursor_mode_;
                     break;
                 }
                 case 0x4a:
@@ -1743,7 +1792,7 @@ private:
                 {
                     typed_array_.clear();
                     read(typed_array_,ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
@@ -1761,7 +1810,8 @@ private:
                             data[i] = binary::byte_swap<int32_t>(data[i]);
                         }
                     }
-                    more_ = visitor.typed_array(jsoncons::span<const int32_t>(data,size), semantic_tag::none, *this, ec);
+                    visitor.typed_array(jsoncons::span<const int32_t>(data,size), semantic_tag::none, *this, ec);
+                    more_ = !cursor_mode_;
                     break;
                 }
                 case 0x4b:
@@ -1769,7 +1819,7 @@ private:
                 {
                     typed_array_.clear();
                     read(typed_array_,ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
@@ -1787,7 +1837,8 @@ private:
                             data[i] = binary::byte_swap<int64_t>(data[i]);
                         }
                     }
-                    more_ = visitor.typed_array(jsoncons::span<const int64_t>(data,size), semantic_tag::none, *this, ec);
+                    visitor.typed_array(jsoncons::span<const int64_t>(data,size), semantic_tag::none, *this, ec);
+                    more_ = !cursor_mode_;
                     break;
                 }
                 case 0x50:
@@ -1795,7 +1846,7 @@ private:
                 {
                     typed_array_.clear();
                     read(typed_array_,ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
@@ -1813,7 +1864,8 @@ private:
                             data[i] = binary::byte_swap<uint16_t>(data[i]);
                         }
                     }
-                    more_ = visitor.typed_array(half_arg, jsoncons::span<const uint16_t>(data,size), semantic_tag::none, *this, ec);
+                    visitor.typed_array(half_arg, jsoncons::span<const uint16_t>(data,size), semantic_tag::none, *this, ec);
+                    more_ = !cursor_mode_;
                     break;
                 }
                 case 0x51:
@@ -1821,7 +1873,7 @@ private:
                 {
                     typed_array_.clear();
                     read(typed_array_,ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
@@ -1839,7 +1891,8 @@ private:
                             data[i] = binary::byte_swap<float>(data[i]);
                         }
                     }
-                    more_ = visitor.typed_array(jsoncons::span<const float>(data,size), semantic_tag::none, *this, ec);
+                    visitor.typed_array(jsoncons::span<const float>(data,size), semantic_tag::none, *this, ec);
+                    more_ = !cursor_mode_;
                     break;
                 }
                 case 0x52:
@@ -1847,7 +1900,7 @@ private:
                 {
                     typed_array_.clear();
                     read(typed_array_,ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
@@ -1866,18 +1919,20 @@ private:
                             data[i] = binary::byte_swap<double>(data[i]);
                         }
                     }
-                    more_ = visitor.typed_array(jsoncons::span<const double>(data,size), semantic_tag::none, *this, ec);
+                    visitor.typed_array(jsoncons::span<const double>(data,size), semantic_tag::none, *this, ec);
+                    more_ = !cursor_mode_;
                     break;
                 }
                 default:
                 {
                     read(bytes_buffer_,ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         more_ = false;
                         return;
                     }
-                    more_ = visitor.byte_string_value(bytes_buffer_, raw_tag_, *this, ec);
+                    visitor.byte_string_value(bytes_buffer_, raw_tag_, *this, ec);
+                    more_ = !cursor_mode_;
                     break;
                 }
             }
@@ -1886,11 +1941,12 @@ private:
         else
         {
             read(bytes_buffer_,ec);
-            if (ec)
+            if (JSONCONS_UNLIKELY(ec))
             {
                 return;
             }
-            more_ = visitor.byte_string_value(bytes_buffer_, semantic_tag::none, *this, ec);
+            visitor.byte_string_value(bytes_buffer_, semantic_tag::none, *this, ec);
+            more_ = !cursor_mode_;
         }
     }
 
@@ -1910,18 +1966,20 @@ private:
         uint8_t info = get_additional_information_value(b);
        
         read_shape(info, ec);   
-        if (ec)
+        if (JSONCONS_UNLIKELY(ec))
         {
             return;
         }
 
         state_stack_.emplace_back(parse_mode::multi_dim, 0);
-        more_ = visitor.begin_multi_dim(shape_, tag, *this, ec);
+        visitor.begin_multi_dim(shape_, tag, *this, ec);
+        more_ = !cursor_mode_;
     }
 
     void produce_end_multi_dim(item_event_visitor& visitor, std::error_code& ec)
     {
-        more_ = visitor.end_multi_dim(*this, ec);
+        visitor.end_multi_dim(*this, ec);
+        more_ = !cursor_mode_;
         state_stack_.pop_back();
     }
 
@@ -1935,7 +1993,7 @@ private:
                 while (true)
                 {
                     auto c = source_.peek();
-                    if (c.eof)
+                    if (JSONCONS_UNLIKELY(c.eof))
                     {
                         ec = cbor_errc::unexpected_eof;
                         more_ = false;
@@ -1948,7 +2006,7 @@ private:
                     else
                     {
                         std::size_t dim = get_size(ec);
-                        if (!more_)
+                        if (JSONCONS_UNLIKELY(ec))
                         {
                             return;
                         }
@@ -1960,14 +2018,14 @@ private:
             default:
             {
                 std::size_t size = get_size(ec);
-                if (!more_)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
                 for (std::size_t i = 0; more_ && i < size; ++i)
                 {
                     std::size_t dim = get_size(ec);
-                    if (!more_)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         return;
                     }
