@@ -54,14 +54,14 @@ public:
 
     template <typename Sourceable>
     basic_csv_cursor(Sourceable&& source, 
-                     const basic_csv_decode_options<CharT>& options = basic_csv_decode_options<CharT>(),
-                     std::function<bool(csv_errc,const ser_context&)> err_handler = default_csv_parsing(),
-                     const Allocator& alloc = Allocator(),
-                     typename std::enable_if<!std::is_constructible<jsoncons::basic_string_view<CharT>,Sourceable>::value>::type* = 0)
+        const basic_csv_decode_options<CharT>& options = basic_csv_decode_options<CharT>(),
+        std::function<bool(csv_errc,const ser_context&)> err_handler = default_csv_parsing(),
+        const Allocator& alloc = Allocator(),
+        typename std::enable_if<!std::is_constructible<jsoncons::basic_string_view<CharT>,Sourceable>::value>::type* = 0)
        : source_(std::forward<Sourceable>(source)),
-         parser_(options,err_handler,alloc),
-         cursor_visitor_(accept_all)
+         parser_(options,err_handler,alloc)
     {
+        parser_.cursor_mode(true);
         if (!done())
         {
             next();
@@ -70,14 +70,14 @@ public:
 
     template <typename Sourceable>
     basic_csv_cursor(Sourceable&& source, 
-                     const basic_csv_decode_options<CharT>& options = basic_csv_decode_options<CharT>(),
-                     std::function<bool(csv_errc,const ser_context&)> err_handler = default_csv_parsing(),
-                     const Allocator& alloc = Allocator(),
-                     typename std::enable_if<std::is_constructible<jsoncons::basic_string_view<CharT>,Sourceable>::value>::type* = 0)
+        const basic_csv_decode_options<CharT>& options = basic_csv_decode_options<CharT>(),
+        std::function<bool(csv_errc,const ser_context&)> err_handler = default_csv_parsing(),
+        const Allocator& alloc = Allocator(),
+        typename std::enable_if<std::is_constructible<jsoncons::basic_string_view<CharT>,Sourceable>::value>::type* = 0)
        : source_(),
-         parser_(options,err_handler,alloc),
-         cursor_visitor_(accept_all)
+         parser_(options,err_handler,alloc)
     {
+        parser_.cursor_mode(true);
         jsoncons::basic_string_view<CharT> sv(std::forward<Sourceable>(source));
         initialize_with_string_view(sv);
     }
@@ -85,52 +85,51 @@ public:
 
     // Constructors that set parse error codes
     template <typename Sourceable>
-    basic_csv_cursor(Sourceable&& source, 
-                     std::error_code& ec)
+    basic_csv_cursor(Sourceable&& source, std::error_code& ec)
         : basic_csv_cursor(std::allocator_arg, Allocator(),
-                           std::forward<Sourceable>(source),
-                           basic_csv_decode_options<CharT>(),
-                           default_csv_parsing(),
-                           ec)
+              std::forward<Sourceable>(source),
+              basic_csv_decode_options<CharT>(),
+              default_csv_parsing(),
+              ec)
     {
     }
 
     template <typename Sourceable>
     basic_csv_cursor(Sourceable&& source, 
-                     const basic_csv_decode_options<CharT>& options,
-                     std::error_code& ec)
+        const basic_csv_decode_options<CharT>& options,
+        std::error_code& ec)
         : basic_csv_cursor(std::allocator_arg, Allocator(),
-                           std::forward<Sourceable>(source),
-                           options,
-                           default_csv_parsing(),
-                           ec)
+              std::forward<Sourceable>(source),
+              options,
+              default_csv_parsing(),
+              ec)
     {
     }
 
     template <typename Sourceable>
     basic_csv_cursor(Sourceable&& source, 
-                     const basic_csv_decode_options<CharT>& options,
-                     std::function<bool(csv_errc,const ser_context&)> err_handler,
-                     std::error_code& ec)
+        const basic_csv_decode_options<CharT>& options,
+        std::function<bool(csv_errc,const ser_context&)> err_handler,
+        std::error_code& ec)
         : basic_csv_cursor(std::allocator_arg, Allocator(),
-                           std::forward<Sourceable>(source),
-                           options,
-                           err_handler,
-                           ec)
+              std::forward<Sourceable>(source),
+              options,
+              err_handler,
+              ec)
     {
     }
 
     template <typename Sourceable>
     basic_csv_cursor(std::allocator_arg_t, const Allocator& alloc, 
-                     Sourceable&& source, 
-                     const basic_csv_decode_options<CharT>& options,
-                     std::function<bool(csv_errc,const ser_context&)> err_handler,
-                     std::error_code& ec,
-                     typename std::enable_if<!std::is_constructible<jsoncons::basic_string_view<CharT>,Sourceable>::value>::type* = 0)
+        Sourceable&& source, 
+        const basic_csv_decode_options<CharT>& options,
+        std::function<bool(csv_errc,const ser_context&)> err_handler,
+        std::error_code& ec,
+        typename std::enable_if<!std::is_constructible<jsoncons::basic_string_view<CharT>,Sourceable>::value>::type* = 0)
        : source_(std::forward<Sourceable>(source)),
-         parser_(options,err_handler,alloc),
-         cursor_visitor_(accept_all)
+         parser_(options,err_handler,alloc)
     {
+        parser_.cursor_mode(true);
         if (!done())
         {
             next(ec);
@@ -139,15 +138,15 @@ public:
 
     template <typename Sourceable>
     basic_csv_cursor(std::allocator_arg_t, const Allocator& alloc, 
-                     Sourceable&& source, 
-                     const basic_csv_decode_options<CharT>& options,
-                     std::function<bool(csv_errc,const ser_context&)> err_handler,
-                     std::error_code& ec,
-                     typename std::enable_if<std::is_constructible<jsoncons::basic_string_view<CharT>,Sourceable>::value>::type* = 0)
+        Sourceable&& source, 
+        const basic_csv_decode_options<CharT>& options,
+        std::function<bool(csv_errc,const ser_context&)> err_handler,
+        std::error_code& ec,
+        typename std::enable_if<std::is_constructible<jsoncons::basic_string_view<CharT>,Sourceable>::value>::type* = 0)
        : source_(),
-         parser_(options,err_handler,alloc),
-         cursor_visitor_(accept_all)
+         parser_(options,err_handler,alloc)
     {
+        parser_.cursor_mode(true);
         jsoncons::basic_string_view<CharT> sv(std::forward<Sourceable>(source));
         initialize_with_string_view(sv, ec);
     }
@@ -216,18 +215,38 @@ public:
     {
         std::error_code ec;
         read_to(visitor, ec);
-        if (ec)
+        if (JSONCONS_UNLIKELY(ec))
         {
             JSONCONS_THROW(ser_error(ec,parser_.line(),parser_.column()));
         }
     }
 
-    void read_to(basic_json_visitor<CharT>& visitor,
-                std::error_code& ec) override
+    void read_to(basic_json_visitor<CharT>& visitor, std::error_code& ec) override
     {
-        if (cursor_visitor_.event().send_json_event(visitor, *this, ec))
+        if (is_begin_container(current().event_type()))
         {
+            parser_.cursor_mode(false);
+            parser_.mark_level(parser_.level());
+            cursor_visitor_.event().send_json_event(visitor, *this, ec);
+            if (JSONCONS_UNLIKELY(ec))
+            {
+                return;
+            }
             read_next(visitor, ec);
+            parser_.cursor_mode(true);
+            parser_.mark_level(0);
+            if (current().event_type() == staj_event_type::begin_object)
+            {
+                cursor_visitor_.end_object(*this);
+            }
+            else
+            {
+                cursor_visitor_.end_array(*this);
+            }
+        }
+        else
+        {
+            cursor_visitor_.event().send_json_event(visitor, *this, ec);
         }
     }
 
@@ -235,7 +254,7 @@ public:
     {
         std::error_code ec;
         next(ec);
-        if (ec)
+        if (JSONCONS_UNLIKELY(ec))
         {
             JSONCONS_THROW(ser_error(ec,parser_.line(),parser_.column()));
         }
@@ -246,11 +265,6 @@ public:
         read_next(ec);
     }
 
-    static bool accept_all(const basic_staj_event<CharT>&, const ser_context&) 
-    {
-        return true;
-    }
-
     const ser_context& context() const override
     {
         return *this;
@@ -273,7 +287,7 @@ public:
 
     friend
     basic_staj_filter_view<CharT> operator|(basic_csv_cursor& cursor, 
-                                      std::function<bool(const basic_staj_event<CharT>&, const ser_context&)> pred)
+        std::function<bool(const basic_staj_event<CharT>&, const ser_context&)> pred)
     {
         return basic_staj_filter_view<CharT>(cursor, pred);
     }
@@ -324,14 +338,14 @@ private:
             if (parser_.source_exhausted())
             {
                 auto s = source_.read_buffer(ec);
-                if (ec) {return;}
+                if (JSONCONS_UNLIKELY(ec)) {return;}
                 if (s.size() > 0)
                 {
                     parser_.update(s.data(),s.size());
                 }
             }
             parser_.parse_some(visitor, ec);
-            if (ec) {return;}
+            if (JSONCONS_UNLIKELY(ec)) {return;}
         }
     }
 };
