@@ -156,24 +156,20 @@ namespace jsonpath {
         private:
 
             allocator_type alloc_;
-            std::size_t line_;
-            std::size_t column_;
-            const char_type* end_input_;
-            const char_type* p_;
+            std::size_t line_{1};
+            std::size_t column_{1};
+            const char_type* end_input_{nullptr};
+            const char_type* p_{nullptr};
 
         public:
             json_location_parser(const allocator_type& alloc = allocator_type())
-                : alloc_(alloc), line_(1), column_(1),
-                  end_input_(nullptr),
-                  p_(nullptr)
+                : alloc_(alloc)
             {
             }
 
             json_location_parser(std::size_t line, std::size_t column, 
                 const allocator_type& alloc = allocator_type())
-                : alloc_(alloc), line_(line), column_(column),
-                  end_input_(nullptr),
-                  p_(nullptr)
+                : alloc_(alloc), line_(line), column_(column)
             {
             }
 
@@ -191,7 +187,7 @@ namespace jsonpath {
             {
                 std::error_code ec;
                 auto result = parse(path, ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     JSONCONS_THROW(jsonpath_error(ec, line_, column_));
                 }
@@ -742,7 +738,7 @@ namespace jsonpath {
             jsonpath::detail::json_location_parser<char,std::allocator<char>> parser;
 
             std::vector<value_type> location = parser.parse(normalized_path, ec);
-            if (ec)
+            if (JSONCONS_UNLIKELY(ec))
             {
                 return basic_json_location();
             }
