@@ -14,7 +14,7 @@
 #include <jsoncons/tag_type.hpp>
 #include <jsoncons/utility/extension_traits.hpp>
 
-#include <jsoncons_ext/jsonpath/expression.hpp>
+#include <jsoncons_ext/jsonpath/token_evaluator.hpp>
 #include <jsoncons_ext/jsonpath/jsonpath_expression.hpp>
 #include <jsoncons_ext/jsonpath/jsonpath_parser.hpp>
 #include <jsoncons_ext/jsonpath/path_node.hpp>
