@@ -1,4 +1,4 @@
-// Copyright 2013-2024 Daniel Parker
+// Copyright 2013-2025 Daniel Parker
 // Distributed under the Boost license, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
@@ -7,14 +7,6 @@
 #ifndef JSONCONS_EXT_MSGPACK_MSGPACK_HPP
 #define JSONCONS_EXT_MSGPACK_MSGPACK_HPP
 
-#include <istream> // std::basic_istream
-#include <memory>
-#include <string>
-#include <type_traits> // std::enable_if
-#include <vector>
-
-#include <jsoncons/config/jsoncons_config.hpp>
-#include <jsoncons/json.hpp>
 #include <jsoncons_ext/msgpack/decode_msgpack.hpp>
 #include <jsoncons_ext/msgpack/encode_msgpack.hpp>
 #include <jsoncons_ext/msgpack/msgpack_cursor.hpp>
