@@ -1,4 +1,4 @@
-// Copyright 2013-2024 Daniel Parker
+// Copyright 2013-2025 Daniel Parker
 // Distributed under the Boost license, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
@@ -7,18 +7,28 @@
 #ifndef JSONCONS_EXT_MSGPACK_MSGPACK_ENCODER_HPP
 #define JSONCONS_EXT_MSGPACK_MSGPACK_ENCODER_HPP
 
+#include <cstddef>
+#include <cstdint>
+#include <cstdlib>
 #include <limits> // std::numeric_limits
 #include <memory>
-#include <string>
+#include <system_error>
 #include <utility> // std::move
 #include <vector>
 
+#include <jsoncons/config/compiler_support.hpp>
 #include <jsoncons/config/jsoncons_config.hpp>
 #include <jsoncons/detail/parse_number.hpp>
+#include <jsoncons/utility/bigint.hpp>
+#include <jsoncons/utility/byte_string.hpp>
 #include <jsoncons/json_exception.hpp>
 #include <jsoncons/json_visitor.hpp>
+#include <jsoncons/ser_context.hpp>
 #include <jsoncons/sink.hpp>
+#include <jsoncons/tag_type.hpp>
 #include <jsoncons/utility/binary.hpp>
+#include <jsoncons/utility/unicode_traits.hpp>
+
 #include <jsoncons_ext/msgpack/msgpack_error.hpp>
 #include <jsoncons_ext/msgpack/msgpack_options.hpp>
 #include <jsoncons_ext/msgpack/msgpack_type.hpp>
@@ -75,7 +85,7 @@ namespace msgpack {
         allocator_type alloc_;
 
         std::vector<stack_item> stack_;
-        int nesting_depth_;
+        int nesting_depth_{0};
     public:
 
         // Noncopyable and nonmoveable
@@ -92,8 +102,7 @@ namespace msgpack {
             const Allocator& alloc = Allocator())
            : sink_(std::forward<Sink>(sink)),
              options_(options),
-             alloc_(alloc),
-             nesting_depth_(0)
+             alloc_(alloc)
         {
         }
 
@@ -125,18 +134,18 @@ namespace msgpack {
             sink_.flush();
         }
 
-        bool visit_begin_object(semantic_tag, const ser_context&, std::error_code& ec) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_begin_object(semantic_tag, const ser_context&, std::error_code& ec) final
         {
             ec = msgpack_errc::object_length_required;
-            return false;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_begin_object(std::size_t length, semantic_tag, const ser_context&, std::error_code& ec) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_begin_object(std::size_t length, semantic_tag, const ser_context&, std::error_code& ec) final
         {
             if (JSONCONS_UNLIKELY(++nesting_depth_ > options_.max_nesting_depth()))
             {
                 ec = msgpack_errc::max_nesting_depth_exceeded;
-                return false;
+                JSONCONS_VISITOR_RETURN;
             } 
             stack_.emplace_back(msgpack_container_type::object, length);
 
@@ -160,10 +169,10 @@ namespace msgpack {
                                       std::back_inserter(sink_));
             }
 
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_end_object(const ser_context&, std::error_code& ec) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_end_object(const ser_context&, std::error_code& ec) final
         {
             JSONCONS_ASSERT(!stack_.empty());
             --nesting_depth_;
@@ -171,31 +180,31 @@ namespace msgpack {
             if (stack_.back().count() < stack_.back().length())
             {
                 ec = msgpack_errc::too_few_items;
-                return false;
+                JSONCONS_VISITOR_RETURN;
             }
             else if (stack_.back().count() > stack_.back().length())
             {
                 ec = msgpack_errc::too_many_items;
-                return false;
+                JSONCONS_VISITOR_RETURN;
             }
 
             stack_.pop_back();
             end_value();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_begin_array(semantic_tag, const ser_context&, std::error_code& ec) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_begin_array(semantic_tag, const ser_context&, std::error_code& ec) final
         {
             ec = msgpack_errc::array_length_required;
-            return false;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_begin_array(std::size_t length, semantic_tag, const ser_context&, std::error_code& ec) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_begin_array(std::size_t length, semantic_tag, const ser_context&, std::error_code& ec) final
         {
             if (JSONCONS_UNLIKELY(++nesting_depth_ > options_.max_nesting_depth()))
             {
                 ec = msgpack_errc::max_nesting_depth_exceeded;
-                return false;
+                JSONCONS_VISITOR_RETURN;
             } 
             stack_.emplace_back(msgpack_container_type::array, length);
             if (length <= 15)
@@ -215,10 +224,10 @@ namespace msgpack {
                 sink_.push_back(jsoncons::msgpack::msgpack_type::array32_type);
                 binary::native_to_big(static_cast<uint32_t>(length),std::back_inserter(sink_));
             }
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_end_array(const ser_context&, std::error_code& ec) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_end_array(const ser_context&, std::error_code& ec) final
         {
             JSONCONS_ASSERT(!stack_.empty());
 
@@ -227,30 +236,31 @@ namespace msgpack {
             if (stack_.back().count() < stack_.back().length())
             {
                 ec = msgpack_errc::too_few_items;
-                return false;
+                JSONCONS_VISITOR_RETURN;
             }
             else if (stack_.back().count() > stack_.back().length())
             {
                 ec = msgpack_errc::too_many_items;
-                return false;
+                JSONCONS_VISITOR_RETURN;
             }
 
             stack_.pop_back();
             end_value();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_key(const string_view_type& name, const ser_context& context, std::error_code& ec) override
+        JSONCONS_VISITOR_RETURN_TYPE visit_key(const string_view_type& name, const ser_context& context, std::error_code& ec) override
         {
-            return visit_string(name, semantic_tag::none, context, ec);
+            visit_string(name, semantic_tag::none, context, ec);
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_null(semantic_tag, const ser_context&, std::error_code&) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_null(semantic_tag, const ser_context&, std::error_code&) final
         {
             // nil
             sink_.push_back(jsoncons::msgpack::msgpack_type::nil_type);
             end_value();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
         void write_timestamp(int64_t seconds, int64_t nanoseconds)
@@ -284,7 +294,7 @@ namespace msgpack {
             }
         }
 
-        bool visit_string(const string_view_type& sv, semantic_tag tag, const ser_context&, std::error_code& ec) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_string(const string_view_type& sv, semantic_tag tag, const ser_context&, std::error_code& ec) final
         {
             switch (tag)
             {
@@ -295,7 +305,7 @@ namespace msgpack {
                     if (!result)
                     {
                         ec = msgpack_errc::invalid_timestamp;
-                        return false;
+                        JSONCONS_VISITOR_RETURN;
                     }
                     write_timestamp(seconds, 0);
                     break;
@@ -351,7 +361,7 @@ namespace msgpack {
                     break;
                 }
             }
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
         void write_string_value(const string_view_type& sv) 
@@ -393,10 +403,10 @@ namespace msgpack {
             }
         }
 
-        bool visit_byte_string(const byte_string_view& b, 
-                               semantic_tag, 
-                               const ser_context&,
-                               std::error_code&) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_byte_string(const byte_string_view& b, 
+            semantic_tag, 
+            const ser_context&,
+            std::error_code&) final
         {
 
             const std::size_t length = b.size();
@@ -425,13 +435,13 @@ namespace msgpack {
             }
 
             end_value();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_byte_string(const byte_string_view& b, 
-                               uint64_t ext_tag, 
-                               const ser_context&,
-                               std::error_code&) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_byte_string(const byte_string_view& b, 
+            uint64_t ext_tag, 
+            const ser_context&,
+            std::error_code&) final
         {
             const std::size_t length = b.size();
             switch (length)
@@ -484,13 +494,13 @@ namespace msgpack {
             }
 
             end_value();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_double(double val, 
-                             semantic_tag,
-                             const ser_context&,
-                             std::error_code&) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_double(double val, 
+            semantic_tag,
+            const ser_context&,
+            std::error_code&) final
         {
             float valf = (float)val;
             if ((double)valf == val)
@@ -509,13 +519,13 @@ namespace msgpack {
             // write double
 
             end_value();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_int64(int64_t val, 
-                         semantic_tag tag, 
-                         const ser_context&,
-                         std::error_code&) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_int64(int64_t val, 
+            semantic_tag tag, 
+            const ser_context&,
+            std::error_code&) final
         {
             switch (tag)
             {
@@ -630,13 +640,13 @@ namespace msgpack {
                 break;
             }
             end_value();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_uint64(uint64_t val, 
-                          semantic_tag tag, 
-                          const ser_context&,
-                          std::error_code&) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_uint64(uint64_t val, 
+            semantic_tag tag, 
+            const ser_context&,
+            std::error_code&) final
         {
             switch (tag)
             {
@@ -716,16 +726,16 @@ namespace msgpack {
                 }
             }
             end_value();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
-        bool visit_bool(bool val, semantic_tag, const ser_context&, std::error_code&) final
+        JSONCONS_VISITOR_RETURN_TYPE visit_bool(bool val, semantic_tag, const ser_context&, std::error_code&) final
         {
             // true and false
             sink_.push_back(static_cast<uint8_t>(val ? jsoncons::msgpack::msgpack_type::true_type : jsoncons::msgpack::msgpack_type::false_type));
 
             end_value();
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
 
         void end_value()
