@@ -58,13 +58,16 @@ class basic_ubjson_parser : public ser_context
     using byte_allocator_type = typename std::allocator_traits<temp_allocator_type>:: template rebind_alloc<uint8_t>;                  
     using parse_state_allocator_type = typename std::allocator_traits<temp_allocator_type>:: template rebind_alloc<parse_state>;                         
 
+    bool more_{true};
+    bool done_{false};
+    int nesting_depth_{0};
+    bool cursor_mode_{false};
+    int mark_level_{0};
+
     Source source_;
     ubjson_decode_options options_;
-    bool more_;
-    bool done_;
     std::basic_string<char,std::char_traits<char>,char_allocator_type> text_buffer_;
     std::vector<parse_state,parse_state_allocator_type> state_stack_;
-    int nesting_depth_;
 public:
     template <typename Sourceable>
         basic_ubjson_parser(Sourceable&& source,
@@ -72,11 +75,8 @@ public:
                           const Allocator& alloc = Allocator())
        : source_(std::forward<Sourceable>(source)), 
          options_(options),
-         more_(true), 
-         done_(false),
          text_buffer_(alloc),
-         state_stack_(alloc),
-         nesting_depth_(0)
+         state_stack_(alloc)
     {
         state_stack_.emplace_back(parse_mode::root,0);
     }
@@ -103,6 +103,26 @@ public:
         reset();
     }
 
+    void cursor_mode(bool value)
+    {
+        cursor_mode_ = value;
+    }
+
+    int level() const
+    {
+        return static_cast<int>(state_stack_.size());
+    }
+
+    int mark_level() const 
+    {
+        return mark_level_;
+    }
+
+    void mark_level(int value)
+    {
+        mark_level_ = value;
+    }
+
     bool done() const
     {
         return done_;
@@ -135,7 +155,7 @@ public:
                     {
                         ++state_stack_.back().index;
                         read_type_and_value(visitor, ec);
-                        if (ec)
+                        if (JSONCONS_UNLIKELY(ec))
                         {
                             return;
                         }
@@ -152,7 +172,7 @@ public:
                     {
                         ++state_stack_.back().index;
                         read_value(visitor, state_stack_.back().type, ec);
-                        if (ec)
+                        if (JSONCONS_UNLIKELY(ec))
                         {
                             return;
                         }
@@ -166,7 +186,7 @@ public:
                 case parse_mode::indefinite_array:
                 {
                     auto c = source_.peek();
-                    if (c.eof)
+                    if (JSONCONS_UNLIKELY(c.eof))
                     {
                         ec = ubjson_errc::unexpected_eof;
                         more_ = false;
@@ -176,7 +196,7 @@ public:
                     {
                         source_.ignore(1);
                         end_array(visitor, ec);
-                        if (ec)
+                        if (JSONCONS_UNLIKELY(ec))
                         {
                             return;
                         }
@@ -190,7 +210,7 @@ public:
                             return;
                         }
                         read_type_and_value(visitor, ec);
-                        if (ec)
+                        if (JSONCONS_UNLIKELY(ec))
                         {
                             return;
                         }
@@ -203,7 +223,7 @@ public:
                     {
                         ++state_stack_.back().index;
                         read_key(visitor, ec);
-                        if (ec)
+                        if (JSONCONS_UNLIKELY(ec))
                         {
                             return;
                         }
@@ -219,7 +239,7 @@ public:
                 {
                     state_stack_.back().mode = parse_mode::map_key;
                     read_type_and_value(visitor, ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         return;
                     }
@@ -231,7 +251,7 @@ public:
                     {
                         ++state_stack_.back().index;
                         read_key(visitor, ec);
-                        if (ec)
+                        if (JSONCONS_UNLIKELY(ec))
                         {
                             return;
                         }
@@ -247,7 +267,7 @@ public:
                 {
                     state_stack_.back().mode = parse_mode::strongly_typed_map_key;
                     read_value(visitor, state_stack_.back().type, ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         return;
                     }
@@ -256,7 +276,7 @@ public:
                 case parse_mode::indefinite_map_key:
                 {
                     auto c = source_.peek();
-                    if (c.eof)
+                    if (JSONCONS_UNLIKELY(c.eof))
                     {
                         ec = ubjson_errc::unexpected_eof;
                         more_ = false;
@@ -266,7 +286,7 @@ public:
                     {
                         source_.ignore(1);
                         end_object(visitor, ec);
-                        if (ec)
+                        if (JSONCONS_UNLIKELY(ec))
                         {
                             return;
                         }
@@ -280,7 +300,7 @@ public:
                             return;
                         }
                         read_key(visitor, ec);
-                        if (ec)
+                        if (JSONCONS_UNLIKELY(ec))
                         {
                             return;
                         }
@@ -292,7 +312,7 @@ public:
                 {
                     state_stack_.back().mode = parse_mode::indefinite_map_key;
                     read_type_and_value(visitor, ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         return;
                     }
@@ -302,7 +322,7 @@ public:
                 {
                     state_stack_.back().mode = parse_mode::accept;
                     read_type_and_value(visitor, ec);
-                    if (ec)
+                    if (JSONCONS_UNLIKELY(ec))
                     {
                         return;
                     }
@@ -346,7 +366,8 @@ private:
         {
             case jsoncons::ubjson::ubjson_type::null_type: 
             {
-                more_ = visitor.null_value(semantic_tag::none, *this, ec);
+                visitor.null_value(semantic_tag::none, *this, ec);
+                more_ = !cursor_mode_;
                 break;
             }
             case jsoncons::ubjson::ubjson_type::no_op_type: 
@@ -355,12 +376,14 @@ private:
             }
             case jsoncons::ubjson::ubjson_type::true_type:
             {
-                more_ = visitor.bool_value(true, semantic_tag::none, *this, ec);
+                visitor.bool_value(true, semantic_tag::none, *this, ec);
+                more_ = !cursor_mode_;
                 break;
             }
             case jsoncons::ubjson::ubjson_type::false_type:
             {
-                more_ = visitor.bool_value(false, semantic_tag::none, *this, ec);
+                visitor.bool_value(false, semantic_tag::none, *this, ec);
+                more_ = !cursor_mode_;
                 break;
             }
             case jsoncons::ubjson::ubjson_type::int8_type: 
@@ -373,7 +396,8 @@ private:
                     return;
                 }
                 int8_t val = binary::big_to_native<int8_t>(buf, sizeof(buf));
-                more_ = visitor.int64_value(val, semantic_tag::none, *this, ec);
+                visitor.int64_value(val, semantic_tag::none, *this, ec);
+                more_ = !cursor_mode_;
                 break;
             }
             case jsoncons::ubjson::ubjson_type::uint8_type: 
@@ -385,7 +409,8 @@ private:
                     more_ = false;
                     return;
                 }
-                more_ = visitor.uint64_value(b, semantic_tag::none, *this, ec);
+                visitor.uint64_value(b, semantic_tag::none, *this, ec);
+                more_ = !cursor_mode_;
                 break;
             }
             case jsoncons::ubjson::ubjson_type::int16_type: 
@@ -398,7 +423,8 @@ private:
                     return;
                 }
                 int16_t val = binary::big_to_native<int16_t>(buf, sizeof(buf));
-                more_ = visitor.int64_value(val, semantic_tag::none, *this, ec);
+                visitor.int64_value(val, semantic_tag::none, *this, ec);
+                more_ = !cursor_mode_;
                 break;
             }
             case jsoncons::ubjson::ubjson_type::int32_type: 
@@ -411,7 +437,8 @@ private:
                     return;
                 }
                 int32_t val = binary::big_to_native<int32_t>(buf, sizeof(buf));
-                more_ = visitor.int64_value(val, semantic_tag::none, *this, ec);
+                visitor.int64_value(val, semantic_tag::none, *this, ec);
+                more_ = !cursor_mode_;
                 break;
             }
             case jsoncons::ubjson::ubjson_type::int64_type: 
@@ -424,7 +451,8 @@ private:
                     return;
                 }
                 int64_t val = binary::big_to_native<int64_t>(buf, sizeof(buf));
-                more_ = visitor.int64_value(val, semantic_tag::none, *this, ec);
+                visitor.int64_value(val, semantic_tag::none, *this, ec);
+                more_ = !cursor_mode_;
                 break;
             }
             case jsoncons::ubjson::ubjson_type::float32_type: 
@@ -437,7 +465,8 @@ private:
                     return;
                 }
                 float val = binary::big_to_native<float>(buf, sizeof(buf));
-                more_ = visitor.double_value(val, semantic_tag::none, *this, ec);
+                visitor.double_value(val, semantic_tag::none, *this, ec);
+                more_ = !cursor_mode_;
                 break;
             }
             case jsoncons::ubjson::ubjson_type::float64_type: 
@@ -450,7 +479,8 @@ private:
                     return;
                 }
                 double val = binary::big_to_native<double>(buf, sizeof(buf));
-                more_ = visitor.double_value(val, semantic_tag::none, *this, ec);
+                visitor.double_value(val, semantic_tag::none, *this, ec);
+                more_ = !cursor_mode_;
                 break;
             }
             case jsoncons::ubjson::ubjson_type::char_type: 
@@ -469,13 +499,14 @@ private:
                     more_ = false;
                     return;
                 }
-                more_ = visitor.string_value(text_buffer_, semantic_tag::none, *this, ec);
+                visitor.string_value(text_buffer_, semantic_tag::none, *this, ec);
+                more_ = !cursor_mode_;
                 break;
             }
             case jsoncons::ubjson::ubjson_type::string_type: 
             {
                 std::size_t length = get_length(ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
@@ -493,13 +524,14 @@ private:
                     more_ = false;
                     return;
                 }
-                more_ = visitor.string_value(jsoncons::basic_string_view<char>(text_buffer_.data(),text_buffer_.length()), semantic_tag::none, *this, ec);
+                visitor.string_value(jsoncons::basic_string_view<char>(text_buffer_.data(),text_buffer_.length()), semantic_tag::none, *this, ec);
+                more_ = !cursor_mode_;
                 break;
             }
             case jsoncons::ubjson::ubjson_type::high_precision_number_type: 
             {
                 std::size_t length = get_length(ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
@@ -512,11 +544,13 @@ private:
                 }
                 if (jsoncons::detail::is_base10(text_buffer_.data(),text_buffer_.length()))
                 {
-                    more_ = visitor.string_value(jsoncons::basic_string_view<char>(text_buffer_.data(),text_buffer_.length()), semantic_tag::bigint, *this, ec);
+                    visitor.string_value(jsoncons::basic_string_view<char>(text_buffer_.data(),text_buffer_.length()), semantic_tag::bigint, *this, ec);
+                    more_ = !cursor_mode_;
                 }
                 else
                 {
-                    more_ = visitor.string_value(jsoncons::basic_string_view<char>(text_buffer_.data(),text_buffer_.length()), semantic_tag::bigdec, *this, ec);
+                    visitor.string_value(jsoncons::basic_string_view<char>(text_buffer_.data(),text_buffer_.length()), semantic_tag::bigdec, *this, ec);
+                    more_ = !cursor_mode_;
                 }
                 break;
             }
@@ -536,7 +570,7 @@ private:
                 break;
             }
         }
-        if (ec)
+        if (JSONCONS_UNLIKELY(ec))
         {
             more_ = false;
         }
@@ -552,7 +586,7 @@ private:
         } 
 
         auto c = source_.peek();
-        if (c.eof)
+        if (JSONCONS_UNLIKELY(c.eof))
         {
             ec = ubjson_errc::unexpected_eof;
             more_ = false;
@@ -569,7 +603,7 @@ private:
                 return;
             }
             c = source_.peek();
-            if (c.eof)
+            if (JSONCONS_UNLIKELY(c.eof))
             {
                 ec = ubjson_errc::unexpected_eof;
                 more_ = false;
@@ -579,7 +613,7 @@ private:
             {
                 source_.ignore(1);
                 std::size_t length = get_length(ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
@@ -590,7 +624,8 @@ private:
                     return;
                 }
                 state_stack_.emplace_back(parse_mode::strongly_typed_array,length,b);
-                more_ = visitor.begin_array(length, semantic_tag::none, *this, ec);
+                visitor.begin_array(length, semantic_tag::none, *this, ec);
+                more_ = !cursor_mode_;
             }
             else
             {
@@ -603,7 +638,7 @@ private:
         {
             source_.ignore(1);
             std::size_t length = get_length(ec);
-            if (ec)
+            if (JSONCONS_UNLIKELY(ec))
             {
                 return;
             }
@@ -614,12 +649,14 @@ private:
                 return;
             }
             state_stack_.emplace_back(parse_mode::array,length);
-            more_ = visitor.begin_array(length, semantic_tag::none, *this, ec);
+            visitor.begin_array(length, semantic_tag::none, *this, ec);
+            more_ = !cursor_mode_;
         }
         else
         {
             state_stack_.emplace_back(parse_mode::indefinite_array,0);
-            more_ = visitor.begin_array(semantic_tag::none, *this, ec);
+            visitor.begin_array(semantic_tag::none, *this, ec);
+            more_ = !cursor_mode_;
         }
     }
 
@@ -627,7 +664,12 @@ private:
     {
         --nesting_depth_;
 
-        more_ = visitor.end_array(*this, ec);
+        visitor.end_array(*this, ec);
+        more_ = !cursor_mode_;
+        if (level() == mark_level_)
+        {
+            more_ = false;
+        }
         state_stack_.pop_back();
     }
 
@@ -641,7 +683,7 @@ private:
         } 
 
         auto c = source_.peek();
-        if (c.eof)
+        if (JSONCONS_UNLIKELY(c.eof))
         {
             ec = ubjson_errc::unexpected_eof;
             more_ = false;
@@ -658,7 +700,7 @@ private:
                 return;
             }
             c = source_.peek();
-            if (c.eof)
+            if (JSONCONS_UNLIKELY(c.eof))
             {
                 ec = ubjson_errc::unexpected_eof;
                 more_ = false;
@@ -668,7 +710,7 @@ private:
             {
                 source_.ignore(1);
                 std::size_t length = get_length(ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
@@ -679,7 +721,8 @@ private:
                     return;
                 }
                 state_stack_.emplace_back(parse_mode::strongly_typed_map_key,length,b);
-                more_ = visitor.begin_object(length, semantic_tag::none, *this, ec);
+                visitor.begin_object(length, semantic_tag::none, *this, ec);
+                more_ = !cursor_mode_;
             }
             else
             {
@@ -691,7 +734,7 @@ private:
         else
         {
             c = source_.peek();
-            if (c.eof)
+            if (JSONCONS_UNLIKELY(c.eof))
             {
                 ec = ubjson_errc::unexpected_eof;
                 more_ = false;
@@ -701,7 +744,7 @@ private:
             {
                 source_.ignore(1);
                 std::size_t length = get_length(ec);
-                if (ec)
+                if (JSONCONS_UNLIKELY(ec))
                 {
                     return;
                 }
@@ -712,12 +755,14 @@ private:
                     return;
                 }
                 state_stack_.emplace_back(parse_mode::map_key,length);
-                more_ = visitor.begin_object(length, semantic_tag::none, *this, ec);
+                visitor.begin_object(length, semantic_tag::none, *this, ec);
+                more_ = !cursor_mode_;
             }
             else
             {
                 state_stack_.emplace_back(parse_mode::indefinite_map_key,0);
-                more_ = visitor.begin_object(semantic_tag::none, *this, ec);
+                visitor.begin_object(semantic_tag::none, *this, ec);
+                more_ = !cursor_mode_;
             }
         }
     }
@@ -725,7 +770,12 @@ private:
     void end_object(json_visitor& visitor, std::error_code& ec)
     {
         --nesting_depth_;
-        more_ = visitor.end_object(*this, ec);
+        visitor.end_object(*this, ec);
+        more_ = !cursor_mode_;
+        if (level() == mark_level_)
+        {
+            more_ = false;
+        }
         state_stack_.pop_back();
     }
 
@@ -860,7 +910,7 @@ private:
     void read_key(json_visitor& visitor, std::error_code& ec)
     {
         std::size_t length = get_length(ec);
-        if (ec)
+        if (JSONCONS_UNLIKELY(ec))
         {
             ec = ubjson_errc::key_expected;
             more_ = false;
@@ -881,7 +931,8 @@ private:
             more_ = false;
             return;
         }
-        more_ = visitor.key(jsoncons::basic_string_view<char>(text_buffer_.data(),text_buffer_.length()), *this, ec);
+        visitor.key(jsoncons::basic_string_view<char>(text_buffer_.data(),text_buffer_.length()), *this, ec);
+        more_ = !cursor_mode_;
     }
 };
 
