@@ -1,4 +1,4 @@
-// Copyright 2013-2024 Daniel Parker
+// Copyright 2013-2025 Daniel Parker
 // Distributed under the Boost license, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
@@ -7,8 +7,7 @@
 #ifndef JSONCONS_EXT_UBJSON_UBJSON_TYPE_HPP
 #define JSONCONS_EXT_UBJSON_UBJSON_TYPE_HPP
 
-#include <memory>
-#include <string>
+#include <cstdint>
 
 #include <jsoncons/config/jsoncons_config.hpp>
 
