@@ -312,72 +312,72 @@ TEST_CASE("Too many and too few items in CBOR map or array")
 
     SECTION("Too many items in array")
     {
-        CHECK(encoder.begin_array(3));
-        CHECK(encoder.bool_value(true));
-        CHECK(encoder.bool_value(false));
-        CHECK(encoder.null_value());
-        CHECK(encoder.begin_array(2));
-        CHECK(encoder.string_value("cat"));
-        CHECK(encoder.string_value("feline"));
-        CHECK(encoder.end_array());
+        encoder.begin_array(3);
+        encoder.bool_value(true);
+        encoder.bool_value(false);
+        encoder.null_value();
+        encoder.begin_array(2);
+        encoder.string_value("cat");
+        encoder.string_value("feline");
+        encoder.end_array();
         REQUIRE_THROWS_WITH(encoder.end_array(), cbor::cbor_error_category_impl().message((int)cbor::cbor_errc::too_many_items).c_str());
         encoder.flush();
     }
     SECTION("Too few items in array")
     {
-        CHECK(encoder.begin_array(5));
-        CHECK(encoder.bool_value(true));
-        CHECK(encoder.bool_value(false));
-        CHECK(encoder.null_value());
-        CHECK(encoder.begin_array(2));
-        CHECK(encoder.string_value("cat"));
-        CHECK(encoder.string_value("feline"));
-        CHECK(encoder.end_array());
+        encoder.begin_array(5);
+        encoder.bool_value(true);
+        encoder.bool_value(false);
+        encoder.null_value();
+        encoder.begin_array(2);
+        encoder.string_value("cat");
+        encoder.string_value("feline");
+        encoder.end_array();
         REQUIRE_THROWS_WITH(encoder.end_array(), cbor::cbor_error_category_impl().message((int)cbor::cbor_errc::too_few_items).c_str());
         encoder.flush();
     }
     SECTION("Too many items in map")
     {
-        CHECK(encoder.begin_object(3));
-        CHECK(encoder.key("a"));
-        CHECK(encoder.bool_value(true));
-        CHECK(encoder.key("b"));
-        CHECK(encoder.bool_value(false));
-        CHECK(encoder.key("c"));
-        CHECK(encoder.null_value());
-        CHECK(encoder.key("d"));
-        CHECK(encoder.begin_array(2));
-        CHECK(encoder.string_value("cat"));
-        CHECK(encoder.string_value("feline"));
-        CHECK(encoder.end_array());
+        encoder.begin_object(3);
+        encoder.key("a");
+        encoder.bool_value(true);
+        encoder.key("b");
+        encoder.bool_value(false);
+        encoder.key("c");
+        encoder.null_value();
+        encoder.key("d");
+        encoder.begin_array(2);
+        encoder.string_value("cat");
+        encoder.string_value("feline");
+        encoder.end_array();
         REQUIRE_THROWS_WITH(encoder.end_object(), cbor::cbor_error_category_impl().message((int)cbor::cbor_errc::too_many_items).c_str());
         encoder.flush();
     }
     SECTION("Too few items in map")
     {
-        CHECK(encoder.begin_object(5));
-        CHECK(encoder.key("a"));
-        CHECK(encoder.bool_value(true));
-        CHECK(encoder.key("b"));
-        CHECK(encoder.bool_value(false));
-        CHECK(encoder.key("c"));
-        CHECK(encoder.null_value());
-        CHECK(encoder.key("d"));
-        CHECK(encoder.begin_array(2));
-        CHECK(encoder.string_value("cat"));
-        CHECK(encoder.string_value("feline"));
-        CHECK(encoder.end_array());
+        encoder.begin_object(5);
+        encoder.key("a");
+        encoder.bool_value(true);
+        encoder.key("b");
+        encoder.bool_value(false);
+        encoder.key("c");
+        encoder.null_value();
+        encoder.key("d");
+        encoder.begin_array(2);
+        encoder.string_value("cat");
+        encoder.string_value("feline");
+        encoder.end_array();
         REQUIRE_THROWS_WITH(encoder.end_object(), cbor::cbor_error_category_impl().message((int)cbor::cbor_errc::too_few_items).c_str());
         encoder.flush();
     }
     SECTION("Just enough items")
     {
-        CHECK(encoder.begin_array(4)); // a fixed length array
-        CHECK(encoder.string_value("foo"));
-        CHECK(encoder.byte_string_value(std::vector<uint8_t>{'P','u','s','s'})); // no suggested conversion
-        CHECK(encoder.string_value("-18446744073709551617", semantic_tag::bigint));
-        CHECK(encoder.string_value("273.15", semantic_tag::bigdec));
-        CHECK(encoder.end_array());
+        encoder.begin_array(4); // a fixed length array
+        encoder.string_value("foo");
+        encoder.byte_string_value(std::vector<uint8_t>{'P','u','s','s'}); // no suggested conversion
+        encoder.string_value("-18446744073709551617", semantic_tag::bigint);
+        encoder.string_value("273.15", semantic_tag::bigdec);
+        encoder.end_array();
         CHECK_FALSE(ec);
         encoder.flush();
     }
@@ -550,28 +550,28 @@ TEST_CASE("test cbor encode with raw tags")
         encoder.flush();
         
         cbor::cbor_bytes_cursor cursor(data);
-        CHECK(cursor.raw_tag() == 0xB1);
-        CHECK(cursor.current().event_type() == jsoncons::staj_event_type::begin_array);
+        CHECK(0xB1 == cursor.raw_tag());
+        CHECK(jsoncons::staj_event_type::begin_array == cursor.current().event_type());
         cursor.next();
-        CHECK(cursor.raw_tag() == 0xC1);
-        CHECK(cursor.current().event_type() == jsoncons::staj_event_type::null_value);
+        CHECK(0xC1 == cursor.raw_tag());
+        CHECK(jsoncons::staj_event_type::null_value == cursor.current().event_type());
         cursor.next();
-        CHECK(cursor.raw_tag() == 0xC2);
-        CHECK(cursor.current().get<bool>() == false);
+        CHECK(0xC2 == cursor.raw_tag());
+        CHECK(false == cursor.current().get<bool>());
         cursor.next();
-        CHECK(cursor.raw_tag() == 0xC3);
-        CHECK(cursor.current().get<uint64_t>() == 1);
+        CHECK(0xC3 == cursor.raw_tag());
+        CHECK(1 == cursor.current().get<uint64_t>());
         cursor.next();
-        CHECK(cursor.raw_tag() == 0xC4);
-        CHECK(cursor.current().get<int64_t>() == -10);
+        CHECK(0xC4 == cursor.raw_tag());
+        CHECK(-10 == cursor.current().get<int64_t>());
         cursor.next();
-        CHECK(cursor.raw_tag() == 0xC5);
-        CHECK(cursor.current().get<double>() == Approx(10.5).epsilon(0.00001));
+        CHECK(0xC5 == cursor.raw_tag());
+        CHECK(Approx(10.5).epsilon(0.00001) == cursor.current().get<double>());
         cursor.next();
-        CHECK(cursor.raw_tag() == 0xC6);
-        CHECK(cursor.current().get<std::vector<uint8_t>>() == std::vector<uint8_t>{0x01,0x02,0x03});
+        CHECK(0xC6 == cursor.raw_tag());
+        CHECK(std::vector<uint8_t>{0x01,0x02,0x03} == cursor.current().get<std::vector<uint8_t>>());
         cursor.next();
-        CHECK(cursor.raw_tag() == 0xD1);
-        CHECK(cursor.current().event_type() == jsoncons::staj_event_type::begin_object);
+        CHECK(0xD1 == cursor.raw_tag());
+        CHECK(jsoncons::staj_event_type::begin_object == cursor.current().event_type());
     }
 }
