@@ -34,25 +34,26 @@ public:
     }
 
 private:
-    bool visit_key(const string_view_type& name,
-                 const ser_context& context,
-                 std::error_code& ec) override
+    JSONCONS_VISITOR_RETURN_TYPE visit_key(const string_view_type& name,
+        const ser_context& context,
+        std::error_code& ec) override
     {
         member_name_ = std::string(name);
         if (member_name_ != "name")
         {
-            return this->destination().key(name, context, ec);
+            this->destination().key(name, context, ec);
+            JSONCONS_VISITOR_RETURN;
         }
         else
         {
-            return true;
+            JSONCONS_VISITOR_RETURN;
         }
     }
 
-    bool visit_string(const string_view_type& s,
-                         semantic_tag tag,
-                         const ser_context& context,
-                         std::error_code&) override
+    JSONCONS_VISITOR_RETURN_TYPE visit_string(const string_view_type& s,
+        semantic_tag tag,
+        const ser_context& context,
+        std::error_code&) override
     {
         if (member_name_ == "name")
         {
@@ -78,7 +79,7 @@ private:
         {
             this->destination().string_value(s, tag, context);
         }
-        return true;
+        JSONCONS_VISITOR_RETURN;
     }
 };
 
@@ -166,10 +167,10 @@ TEST_CASE("test_chained_filters")
 
     j.dump(filter1);
     ojson j2 = decoder.get_result();
-    CHECK(j2.size() == 4);
-    CHECK(j2["first"] == 1);
-    CHECK(j2["second"] == 2);
-    CHECK(j2["third"] == 3);
-    CHECK(j2["fourth"] == 4);
+    CHECK(4 == j2.size());
+    CHECK(1 == j2["first"]);
+    CHECK(2 == j2["second"]);
+    CHECK(3 == j2["third"]);
+    CHECK(4 == j2["fourth"]);
 }
 
