@@ -1,17 +1,17 @@
-// Copyright 2013-2024 Daniel Parker
+// Copyright 2013-2025 Daniel Parker
 // Distributed under Boost license
 
 #include <jsoncons/json.hpp>
 #include <jsoncons/json_reader.hpp>
-#include <catch/catch.hpp>
 #include <sstream>
 #include <vector>
 #include <utility>
 #include <ctime>
+#include <catch/catch.hpp>
 
 using namespace jsoncons;
 
-template <class Source>
+template <typename Source>
 typename std::enable_if<extension_traits::is_byte<typename Source::value_type>::value,std::size_t>::type
 read_json(Source& source, char* buffer, std::size_t capacity, unicode_traits::encoding_kind& encoding)
 {
